package com.sztzjy.money_management.controller;

import cn.hutool.core.util.IdUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.nimbusds.jose.shaded.gson.Gson;
import com.sztzjy.money_management.annotation.AnonymousAccess;
import com.sztzjy.money_management.entity.*;
import com.sztzjy.money_management.entity.dto.*;
import com.sztzjy.money_management.entity.vo.ObjCaseScoreVo;
import com.sztzjy.money_management.mapper.*;
import com.sztzjy.money_management.service.CaseService;
import com.sztzjy.money_management.service.ScoreRankService;
import com.sztzjy.money_management.service.TrainingService;
import com.sztzjy.money_management.util.ResultEntity;
import com.sztzjy.money_management.util.excel.FilePortUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.python.core.AstList;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.FileSystemResource;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.math.BigDecimal;
import java.util.*;

@RestController
@Api(tags = "成绩相关")
@RequestMapping("api/GradeController")
public class GradeController {
    @Autowired
    StuTrainingMapper trainingMapper;
    @Autowired
    StudentScoreWeightMapper studentScoreWeightMapper;
    @Autowired
    TrainingReportMapper trainingReportMapper;
    @Autowired
    TrainingService trainingService;
    @Autowired
    StuTheoryRecordMapper stuTheoryRecordMapper;
    @Autowired
    TotalWeightMapper totalWeightMapper;
    @Autowired
    ScoreRankMapper scoreRankMapper;
    @Autowired
    ScoreRankService scoreRankService;
    @Autowired
    ClientWeightMapper clientWeightMapper;
    @Autowired
    ChapterMapper chapterMapper;
    @Value("${file.path}")
    private String filePath;
    @Autowired
    SynthesisPlanClientMapper synthesisPlanClientMapper;
    @Autowired
    CaseService caseService;
    @Autowired
    CaseInfoMapper caseInfoMapper;
    @Autowired
    ComCaseMapper comCaseMapper;
    @Autowired
    CaseFinancialRatiosMapper caseFinancialRatiosMapper;
    @Autowired
    SynthesisPlanScoreMapper synthesisPlanScoreMapper;
    @Autowired
    CaseCashPlanMapper caseCashPlanMapper;
    @Autowired
    CaseLifeEducationPlanMapper caseLifeEducationPlanMapper;
    @Autowired
    CaseConsumptionPlanMapper caseConsumptionPlanMapper;
    @Autowired
    CaseStartAnUndertakingPlanMapper caseStartAnUndertakingPlanMapper;
    @Autowired
    CaseRetirementPlanMapper caseRetirementPlanMapper;
    @Autowired
    CaseInsurancePlanMapper caseInsurancePlanMapper;
    @Autowired
    CaseTaxPlanMapper caseTaxPlanMapper;
    @Autowired
    CaseDistributionOfPropertyMapper caseDistributionOfPropertyMapper;
    @Autowired
    CaseHeritageMapper caseHeritageMapper;
    @Autowired
    CaseInvestmentPlanMapper caseInvestmentPlanMapper;
    @Autowired
    UserInfoMapper userInfoMapper;

    @AnonymousAccess
    @ApiOperation("实训演练-学习成绩得分情况展示")
    @PostMapping("getScoreInfo")
    public ResultEntity<StuTrainingDto> getScoreInfo(@RequestParam String userId,
                                                     @RequestParam String chapterId,
                                                     @RequestParam String schoolId) {
        StudentScoreWeight studentScoreWeight = new StudentScoreWeight();
        //先根据schoolID和chapterId查出学习成绩权重表数据
        StudentScoreWeightExample studentScoreWeightExample = new StudentScoreWeightExample();
        studentScoreWeightExample.createCriteria().andSchoolIdEqualTo(schoolId).andChapterIdEqualTo(chapterId);
        List<StudentScoreWeight> studentScoreWeights = studentScoreWeightMapper.selectByExample(studentScoreWeightExample);
        if (studentScoreWeights.isEmpty()) {  //设置默认权重与计分规则
            //计分规则
            studentScoreWeight.setSummaryOfKnowledgeScore(BigDecimal.valueOf(20));
            studentScoreWeight.setResourceLearningScore(BigDecimal.valueOf(20));
            studentScoreWeight.setLearningAssessmentScore(BigDecimal.valueOf(10));
            studentScoreWeight.setExperimentalTrainingScore(BigDecimal.valueOf(10));

            //权重
            studentScoreWeight.setSummaryOfKnowledgeWeight(BigDecimal.valueOf(0.2));
            studentScoreWeight.setResourceLearningWeight(BigDecimal.valueOf(0.2));
            studentScoreWeight.setLearningAssessmentWeight(BigDecimal.valueOf(0.2));
            studentScoreWeight.setExperimentalTrainingWeight(BigDecimal.valueOf(0.2));
            studentScoreWeight.setReportWeight(BigDecimal.valueOf(0.2));
        } else {
            studentScoreWeight = studentScoreWeights.get(0);
        }
        //再根据userid和chapterId查出学生实训表中的数据
        StuTrainingExample stuTrainingExample = new StuTrainingExample();
        stuTrainingExample.createCriteria().andUserIdEqualTo(userId).andChapterIdEqualTo(chapterId);
        List<StuTraining> stuTrainings = trainingMapper.selectByExample(stuTrainingExample);
        StuTraining stuTraining = new StuTraining();
        if (!stuTrainings.isEmpty()) {
            stuTraining = stuTrainings.get(0);
        }
        //进行整合
        StuTrainingDto stuTrainingDto = new StuTrainingDto();
        //设置返回对象计分规则
        stuTrainingDto.setKnowledgeSummaryDefaultRule(studentScoreWeight.getSummaryOfKnowledgeScore());
        stuTrainingDto.setResourceLearningDefaultRule(studentScoreWeight.getResourceLearningScore());
        stuTrainingDto.setLearningEvalDefaultRule(studentScoreWeight.getLearningAssessmentScore());
        stuTrainingDto.setExpTrainingDefautRule(studentScoreWeight.getExperimentalTrainingScore());
        //设置返回对象权重
        stuTrainingDto.setKnowledgeSummaryWeight(studentScoreWeight.getSummaryOfKnowledgeWeight());
        stuTrainingDto.setResourceLearningWeight(studentScoreWeight.getResourceLearningWeight());
        stuTrainingDto.setLearningEvalWeight(studentScoreWeight.getLearningAssessmentWeight());
        stuTrainingDto.setExpTrainingWeight(studentScoreWeight.getExperimentalTrainingWeight());
        stuTrainingDto.setReportWeight(studentScoreWeight.getReportWeight());
        //设置返回对象完成情况
        stuTrainingDto.setKnowledgeSummaryCompleteStatus(stuTraining.getKnowledgeSummaryCompleteStatus() == null ? 0 : stuTraining.getKnowledgeSummaryCompleteStatus());
        stuTrainingDto.setResourceLearningCompleteStatus(stuTraining.getResourceLearningCompleteStatus() == null ? 0 : stuTraining.getResourceLearningCompleteStatus());
        stuTrainingDto.setLearningEvalCompleteStatus(stuTraining.getLearningEvalCompleteStatus());
        stuTrainingDto.setExpTrainingCompleteStatus(stuTraining.getExpTrainingCompleteStatus());
        TrainingReportExample trainingReportExample = new TrainingReportExample();
        trainingReportExample.createCriteria().andChapterIdEqualTo(chapterId).andUserIdEqualTo(userId);
        //设置返回对象得分  根据学生实训表完成情况 设置得分
        stuTrainingDto.setKnowledgeSummaryScore(studentScoreWeight.getSummaryOfKnowledgeScore().multiply(BigDecimal.valueOf(stuTraining.getKnowledgeSummaryCompleteStatus() == null ? 0 : stuTraining.getKnowledgeSummaryCompleteStatus())));
        stuTrainingDto.setResourceLearningScore(studentScoreWeight.getResourceLearningScore().multiply(BigDecimal.valueOf(stuTraining.getResourceLearningCompleteStatus() == null ? 0 : stuTraining.getResourceLearningCompleteStatus())));
        if(stuTrainingDto.getLearningEvalCompleteStatus()==null){
            stuTrainingDto.setLearningEvalScore(BigDecimal.valueOf(0));
        }else {
            stuTrainingDto.setLearningEvalScore(BigDecimal.valueOf(100).subtract(studentScoreWeight.getLearningAssessmentScore().multiply(BigDecimal.valueOf(stuTraining.getLearningEvalCompleteStatus()))));
            if (stuTrainingDto.getLearningEvalScore().compareTo(BigDecimal.valueOf(0)) < 0) {
                stuTrainingDto.setLearningEvalScore(BigDecimal.valueOf(0));
            }
        }
        if(stuTrainingDto.getExpTrainingCompleteStatus()==null){
            stuTrainingDto.setCaseStuScore(BigDecimal.valueOf(0));
        }else {
            stuTrainingDto.setCaseStuScore(BigDecimal.valueOf(100).subtract(studentScoreWeight.getExperimentalTrainingScore().multiply(BigDecimal.valueOf(stuTraining.getExpTrainingCompleteStatus() == null ? 0 : stuTraining.getExpTrainingCompleteStatus()))));
            if (stuTrainingDto.getCaseStuScore().compareTo(BigDecimal.valueOf(0)) < 0) {
                stuTrainingDto.setCaseStuScore(BigDecimal.valueOf(0));
            }
        }
        if (stuTrainingDto.getKnowledgeSummaryScore().compareTo(BigDecimal.valueOf(100)) > 0) {
            stuTrainingDto.setKnowledgeSummaryScore(BigDecimal.valueOf(100));
        }
        if (stuTrainingDto.getResourceLearningScore().compareTo(BigDecimal.valueOf(100)) > 0) {
            stuTrainingDto.setResourceLearningScore(BigDecimal.valueOf(100));
        }


        //设置实验报告情况及得分
        List<TrainingReport> trainingReports = trainingReportMapper.selectByExample(trainingReportExample);
        if (trainingReports.isEmpty()) {
            stuTrainingDto.setReportCompleteStatus("未提交");
            stuTrainingDto.setReportScore(BigDecimal.valueOf(0));
        } else {
            stuTrainingDto.setReportCompleteStatus("已提交");
            stuTrainingDto.setReportScore(trainingReports.get(0).getTeacherScore());
        }
        if("11".equals(chapterId)){  //特殊处理财产分配
            stuTrainingDto.setExpTrainingCompleteStatus(null);
            stuTrainingDto.setExpTrainingDefautRule(null);
            if(stuTraining.getExpTrainingCompleteStatus()==null){
                stuTrainingDto.setCaseStuScore(BigDecimal.valueOf(0));
            }else if(stuTraining.getExpTrainingCompleteStatus()<=100){
                stuTrainingDto.setCaseStuScore(BigDecimal.valueOf(stuTraining.getExpTrainingCompleteStatus()));
            }else {
                stuTrainingDto.setCaseStuScore(BigDecimal.valueOf(100));
            }
        }

        return new ResultEntity<>(HttpStatus.OK, "得分情况展示", stuTrainingDto);

    }

    @GetMapping("/returnTrainingId")
    @ApiOperation("第一次进入时创建学生实训对象")
    @AnonymousAccess
    public ResultEntity<String> returnTrainingId(@RequestParam String chapterId,
                                                 @RequestParam String userId) {
        StuTrainingExample stuTrainingExample = new StuTrainingExample();
        stuTrainingExample.createCriteria().andChapterIdEqualTo(chapterId).andUserIdEqualTo(userId);
        List<StuTrainingWithBLOBs> list = trainingMapper.selectByExampleWithBLOBs(stuTrainingExample);
        Chapter chapter = chapterMapper.selectByPrimaryKey(chapterId);
        if (list.isEmpty()) { //该用户第一次进入这个章节的知识概要 新增
            StuTrainingWithBLOBs stuTraining = new StuTrainingWithBLOBs();
            String uuid = IdUtil.randomUUID();
            stuTraining.setTrainingId(uuid);
            stuTraining.setChapterId(chapterId);
            stuTraining.setChapterName(chapter.getChapterName());
            stuTraining.setUserId(userId);
            trainingMapper.insert(stuTraining);
            return new ResultEntity<>(uuid);
        } else {
            return new ResultEntity<>(list.get(0).getTrainingId());
        }
    }


    //学生端 客户案例成绩展示
    @AnonymousAccess
    @ApiOperation("学生端-成绩中心-客户案例成绩展示")
    @PostMapping("selectStuClientCaseScoreList")
    public ResultEntity<List<StuClientCaseScoreDto>> selectStuClientCaseScoreList(@RequestParam String userId,
                                                                                  @RequestParam String schoolId) {
        List<StuClientCaseScoreDto> list = trainingService.selectStuClientCaseScoreList(userId, schoolId);
        return new ResultEntity<>(HttpStatus.OK, "学生端-成绩中心-客户案例成绩展示", list);
    }

    //学生端 综合规划成绩
    @AnonymousAccess
    @ApiOperation("学生端-成绩中心-综合规划成绩展示")
    @PostMapping("selectStuSynthesisPlanScoreList")
    public ResultEntity<List<StuSynthesisPlanScoreDto>> selectStuSynthesisPlanScoreList(@RequestParam String userId,
                                                                                        @RequestParam String schoolId) {
        List<StuSynthesisPlanScoreDto> list = trainingService.selectStuSynthesisPlanScoreList(userId, schoolId);
        return new ResultEntity<>(HttpStatus.OK, "学生端-成绩中心-综合规划成绩展示", list);
    }

    //学生端 综合规划审核报告展示  同综合规划-规划报告-审核报告

    //学生端 理论考核成绩
    @AnonymousAccess
    @ApiOperation("学生端-成绩中心-理论考核成绩展示")
    @PostMapping("selectStuTheoryScore")
    public ResultEntity<StuTheoryRecordDto> selectStuTheoryRecord(@RequestParam String userId, @RequestParam String schoolId) {
        StuTheoryRecord stuTheoryRecord = stuTheoryRecordMapper.selectByPrimaryKey(userId);
        if (stuTheoryRecord == null) {
            return new ResultEntity<>(HttpStatus.OK, "暂无数据", null);
        }
        StuTheoryRecordDto stuTheoryRecordDto = new StuTheoryRecordDto();
        stuTheoryRecordDto.setExamCount(stuTheoryRecord.getExamCount());
        stuTheoryRecordDto.setHighestScore(stuTheoryRecord.getHighestScore());
        stuTheoryRecordDto.setLowestScore(stuTheoryRecord.getLowestScore());
        stuTheoryRecordDto.setAverageScore(stuTheoryRecord.getAverageScore());

        //设置权重
        TotalWeight totalWeight = totalWeightMapper.selectByPrimaryKey(schoolId);
        if (totalWeight == null) {
            stuTheoryRecordDto.setWeight(BigDecimal.valueOf(0.2));
        } else {
            stuTheoryRecordDto.setWeight(totalWeight.getLlkh());
        }
        return new ResultEntity<>(HttpStatus.OK, "学生端-成绩中心-理论考核成绩展示", stuTheoryRecordDto);
    }


    //学生端 成绩总览
    @AnonymousAccess
    @ApiOperation("学生端-成绩中心-成绩总览展示")
    @PostMapping("selectStuTotalScore")
    public ResultEntity<List<StuTotalScoreDto>> selectStuTotalScore(@RequestParam String userId, @RequestParam String schoolId) {
        List<StuTotalScoreDto> returnDtoList = new ArrayList<>();
        //客户案例数据
        StuTotalScoreDto clientCaseReturnDto = new StuTotalScoreDto();
        List<StuClientCaseScoreDto> clientCaseList = trainingService.selectStuClientCaseScoreList(userId, schoolId);
        clientCaseReturnDto.setModelName("客户案例成绩");
        BigDecimal clientCaseTrainingTime = BigDecimal.ZERO;
        BigDecimal clientCaseTotalTrainingScore = BigDecimal.ZERO;
        clientCaseReturnDto.setTrainingScore(BigDecimal.ZERO);
        clientCaseReturnDto.setTrainingTotalScore(BigDecimal.ZERO);
        clientCaseReturnDto.setUseTime(BigDecimal.ZERO);
        //根据学校iD获取成绩总览权重
        TotalWeight totalWeight = totalWeightMapper.selectByPrimaryKey(schoolId);
        if (totalWeight == null) {
            totalWeight = new TotalWeight();
            totalWeight.setKhan(BigDecimal.valueOf(0.4));
            totalWeight.setZhgh(BigDecimal.valueOf(0.4));
            totalWeight.setLlkh(BigDecimal.valueOf(0.2));
        }
        clientCaseReturnDto.setScoreWight(totalWeight.getKhan());
        if (clientCaseList.isEmpty()) {
            returnDtoList.add(clientCaseReturnDto);
        } else {
            for (int i = 0; i < clientCaseList.size(); i++) {
                StuClientCaseScoreDto clientCase = clientCaseList.get(i);
                BigDecimal trainingTime = clientCase.getTrainingTime() == null ? BigDecimal.ZERO : clientCase.getTrainingTime();
                BigDecimal totalScore = clientCase.getTrainingTotalScore() == null ? BigDecimal.ZERO : clientCase.getTrainingScore();
                clientCaseTrainingTime = clientCaseTrainingTime.add(trainingTime);
                clientCaseTotalTrainingScore = clientCaseTotalTrainingScore.add(totalScore);
            }
            clientCaseReturnDto.setTrainingTotalScore(clientCaseTotalTrainingScore);
            clientCaseReturnDto.setUseTime(clientCaseTrainingTime);
            clientCaseReturnDto.setTrainingScore(clientCaseTotalTrainingScore.multiply(totalWeight.getKhan()));
            returnDtoList.add(clientCaseReturnDto);
        }

        //综合规划成绩
        StuTotalScoreDto synthesisPlanReturnDto = new StuTotalScoreDto();
        synthesisPlanReturnDto.setModelName("综合规划成绩");
        ScoreRankExample scoreRankExample = new ScoreRankExample();
        scoreRankExample.createCriteria().andUseridEqualTo(userId).andUpdateDateEqualTo(scoreRankService.getNowDate());
        List<ScoreRank> scoreRankList = scoreRankMapper.selectByExample(scoreRankExample);
        if (scoreRankList.isEmpty()) {
            synthesisPlanReturnDto.setTrainingScore(BigDecimal.ZERO);
            synthesisPlanReturnDto.setUseTime(BigDecimal.ZERO);
            synthesisPlanReturnDto.setTrainingTotalScore(BigDecimal.ZERO);
        } else {
            ScoreRank scoreRank = scoreRankList.get(0);
            Integer totalUseTime = synthesisPlanClientMapper.getTotalUseTime(userId);
            if (totalUseTime==null){
                totalUseTime=0;
            }
            synthesisPlanReturnDto.setUseTime(BigDecimal.valueOf(totalUseTime));
            synthesisPlanReturnDto.setTrainingScore(scoreRank.getZhghScore().multiply(totalWeight.getZhgh()));
            synthesisPlanReturnDto.setTrainingTotalScore(scoreRank.getZhghScore());
        }
        synthesisPlanReturnDto.setScoreWight(totalWeight.getZhgh());
        returnDtoList.add(synthesisPlanReturnDto);

        //理论考试成绩
        StuTotalScoreDto theoryReturnDto = new StuTotalScoreDto();
        theoryReturnDto.setModelName("理论考试成绩");
        if (scoreRankList.isEmpty()) {
            theoryReturnDto.setTrainingScore(BigDecimal.ZERO);
            theoryReturnDto.setUseTime(BigDecimal.ZERO);
            theoryReturnDto.setTrainingTotalScore(BigDecimal.ZERO);
        } else {
            StuTheoryRecord stuTheoryRecord = stuTheoryRecordMapper.selectByPrimaryKey(userId);
            if (stuTheoryRecord == null) {
                theoryReturnDto.setTrainingScore(BigDecimal.ZERO);
                theoryReturnDto.setUseTime(BigDecimal.ZERO);
                theoryReturnDto.setTrainingTotalScore(BigDecimal.ZERO);
            } else {
                theoryReturnDto.setUseTime(stuTheoryRecord.getTotalDuration());
                theoryReturnDto.setTrainingTotalScore(stuTheoryRecord.getAverageScore());
                theoryReturnDto.setTrainingTotalScore(stuTheoryRecord.getAverageScore().multiply(totalWeight.getLlkh()));
            }
        }
        theoryReturnDto.setScoreWight(totalWeight.getLlkh());
        returnDtoList.add(theoryReturnDto);
        return new ResultEntity<>(HttpStatus.OK, "学生端-成绩中心-成绩总览展示成功", returnDtoList);
    }


    //教师端 客户案例成绩查询 成绩总览成绩查询
    @AnonymousAccess
    @ApiOperation("教师端-成绩中心-成绩总览展示")
    @PostMapping("selectTeaSummaryOfResults")
    public ResultEntity<PageInfo<ScoreRank>> selectTeaSummaryOfResults(@RequestParam String schoolId, @RequestParam Integer index, @RequestParam Integer size,
                                                                       @RequestParam(required = false) String name,
                                                                       @RequestParam(required = false) String studentId,
                                                                       @RequestParam(required = false) String classId) {
        PageHelper.startPage(index, size);
        ScoreRankExample scoreRankExample = new ScoreRankExample();
        ScoreRankExample.Criteria criteria = scoreRankExample.createCriteria();
        criteria.andSchoolIdEqualTo(schoolId).andUpdateDateEqualTo(scoreRankService.getNowDate());
        if (StringUtils.isNotBlank(name)) {
            criteria.andNameEqualTo(name);
        }
        if (StringUtils.isNotBlank(studentId)) {
            criteria.andSchoolIdEqualTo(studentId);
        }
        if (StringUtils.isNotBlank(classId)) {
            criteria.andClassIdEqualTo(classId);
        }
//        scoreRankExample.setOrderByClause("khal_score+zhgh_score+llkh_score desc");
        scoreRankExample.setOrderByClause("total_rank asc");
        List<ScoreRank> scoreRankList = scoreRankMapper.selectByExample(scoreRankExample);
        //查询成绩总览权重
        TotalWeight totalWeight = totalWeightMapper.selectByPrimaryKey(schoolId);
        if (totalWeight == null) {
            totalWeight = new TotalWeight();
            totalWeight.setKhan(BigDecimal.valueOf(0.4));
            totalWeight.setZhgh(BigDecimal.valueOf(0.4));
            totalWeight.setLlkh(BigDecimal.valueOf(0.2));
        }
        //设置总成绩
        for (int i = 0; i < scoreRankList.size(); i++) {
            ScoreRank scoreRank = scoreRankList.get(i);
            scoreRank.setTotalScore((totalWeight.getKhan().multiply(scoreRank.getKhalScore()))
                    .add(totalWeight.getZhgh().multiply(scoreRank.getZhghScore()))
                    .add(totalWeight.getLlkh().multiply(scoreRank.getLlkhScore())));
        }
        PageInfo pageInfo = new PageInfo(scoreRankList);
        return new ResultEntity<PageInfo<ScoreRank>>(pageInfo);
    }

    @AnonymousAccess
    @ApiOperation("教师端-成绩中心-成绩总览权重查询")
    @PostMapping("seleteTotalWeight")
    public ResultEntity<TotalWeight> seleteTotalWeight(@RequestBody String schoolId) {
        TotalWeight totalWeight = totalWeightMapper.selectByPrimaryKey(schoolId);
        if (totalWeight == null) {
            totalWeight = new TotalWeight();
            totalWeight.setSchoolId(schoolId);
            totalWeight.setKhan(BigDecimal.valueOf(0.4));
            totalWeight.setZhgh(BigDecimal.valueOf(0.4));
            totalWeight.setLlkh(BigDecimal.valueOf(0.2));
        }
        return new ResultEntity<>(totalWeight);
    }

    @AnonymousAccess
    @ApiOperation("教师端-成绩中心-成绩总览权重设置")
    @PostMapping("setTotalWeight")
    public Boolean setTotalWeight(@RequestBody TotalWeight totalWeight) {
        TotalWeight totalWeight1 = totalWeightMapper.selectByPrimaryKey(totalWeight.getSchoolId());
        if (totalWeight1 == null) {
            int insert = totalWeightMapper.insert(totalWeight);
            return insert == 1;
        } else {
            int i = totalWeightMapper.updateByPrimaryKey(totalWeight);
            return i == 1;
        }
    }


    //教师端 成绩总览导出
    @AnonymousAccess
    @GetMapping("/exportSummaryOfResults")
    @ApiOperation("教师端--成绩总览导出页面导出")
    public void exportSummaryOfResults(HttpServletResponse response,
                                       @RequestParam String schoolId) {
        List<ScoreRank> scoreRankList = scoreRankService.selectExportScoreRankList(schoolId);
        //导出的表名
        String title = "成绩总览";
        //表中第一行表头字段
        String[] headers = {"排名", "姓名", "学号", "总成绩", "班级", "客户案例成绩", "综合规划成绩", "理论考核成绩"};

        //具体需要写入excel需要哪些字段，这些字段取自UserReward类，也就是上面的实际数据结果集的泛型
        List<String> listColumn = Arrays.asList("rankId", "name", "studentId", "totalScore", "className", "khalScore", "zhghScore", "llkhScore");
        try {
            FilePortUtil.exportExcel(response, title, headers, scoreRankList, listColumn);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    //客户案例成绩查询
    @AnonymousAccess
    @ApiOperation("教师端-成绩中心-客户案例展示")
    @PostMapping("selectTeaClientCaseScoreList")
    public ResultEntity<PageInfo<ScoreRank>> selectTeaClientCaseScoreList(@RequestParam String schoolId, @RequestParam Integer index, @RequestParam Integer size,
                                                                          @RequestParam(required = false) String name,
                                                                          @RequestParam(required = false) String studentId,
                                                                          @RequestParam(required = false) String classId) {
        PageHelper.startPage(index, size);
        ScoreRankExample scoreRankExample = new ScoreRankExample();
        ScoreRankExample.Criteria criteria = scoreRankExample.createCriteria();
        criteria.andSchoolIdEqualTo(schoolId).andUpdateDateEqualTo(scoreRankService.getNowDate());
        if (StringUtils.isNotBlank(name)) {
            criteria.andNameEqualTo(name);
        }
        if (StringUtils.isNotBlank(studentId)) {
            criteria.andSchoolIdEqualTo(studentId);
        }
        if (StringUtils.isNotBlank(classId)) {
            criteria.andClassIdEqualTo(classId);
        }
        scoreRankExample.setOrderByClause("khal_rank asc");
        List<ScoreRank> scoreRankList = scoreRankMapper.selectByExample(scoreRankExample);

        //设置总成绩
//        for (int i = 0; i < scoreRankList.size(); i++) {
//            ScoreRank scoreRank = scoreRankList.get(i);
//            scoreRank.setTotalScore(scoreRank.getZhghScore());
//        }
        PageInfo pageInfo = new PageInfo(scoreRankList);
        return new ResultEntity<PageInfo<ScoreRank>>(pageInfo);
    }

    //教师端 客户案例导出
    @AnonymousAccess
    @GetMapping("/exportClientCase")
    @ApiOperation("教师端--客户案例导出页面导出")
    public void exportClientCase(HttpServletResponse response,
                                 @RequestParam String schoolId) {
        List<ScoreRank> scoreRankList = scoreRankService.selectExportScoreRankList(schoolId);
        //导出的表名
        String title = IdUtil.simpleUUID();
        //表中第一行表头字段
        String[] headers = {"排名", "姓名", "学号", "班级", "总成绩", "风险测评分数", "财务分析分数", "现金规划分数", "教育规划分数", "消费规划分数", "创业规划分数", "退休规划分数", "保险规划分数", "投资规划分数", "税务筹划分数", "财产分配分数", "财产传承分数"};
        //具体需要写入excel需要哪些字段，这些字段取自UserReward类，也就是上面的实际数据结果集的泛型
        List<String> listColumn = Arrays.asList("rankId", "name", "studentId", "className", "khalScore", "fxcp", "cwfx", "xjgh", "jygh", "xfgh", "cygh", "txgh", "bxgh", "tzgh", "swch", "ccfp", "cccc");
        try {
            FilePortUtil.exportExcel(response, title, headers, scoreRankList, listColumn);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    //教师端 客户案例权重查询
    @AnonymousAccess
    @ApiOperation("教师端-成绩中心-客户案例权重查询")
    @PostMapping("seleteClientWeight")
    public ResultEntity<ClientWeight> seleteClientWeight(@RequestBody String schoolId) {
        ClientWeight clientWeight = clientWeightMapper.selectByPrimaryKey(schoolId);
        if (clientWeight == null) {
            clientWeight = new ClientWeight();
            clientWeight.setFxcp(BigDecimal.valueOf(0.05));
            clientWeight.setCwfx(BigDecimal.valueOf(0.20));
            clientWeight.setXjgh(BigDecimal.valueOf(0.05));
            clientWeight.setJygh(BigDecimal.valueOf(0.1));
            clientWeight.setXfgh(BigDecimal.valueOf(0.1));
            clientWeight.setCygh(BigDecimal.valueOf(0.05));
            clientWeight.setTxgh(BigDecimal.valueOf(0.1));
            clientWeight.setBxgh(BigDecimal.valueOf(0.15));
            clientWeight.setTzgh(BigDecimal.valueOf(0.05));
            clientWeight.setSwch(BigDecimal.valueOf(0.05));
            clientWeight.setCcfp(BigDecimal.valueOf(0.05));
            clientWeight.setCccc(BigDecimal.valueOf(0.05));
        }
        return new ResultEntity<>(clientWeight);
    }

    //教师端 客户案例权重设置
    @AnonymousAccess
    @ApiOperation("教师端-成绩中心-客户案例权重设置")
    @PostMapping("setClientWeight")
    public Boolean setClientWeight(@RequestBody ClientWeight clientWeight) {
        ClientWeight clientWeight1 = clientWeightMapper.selectByPrimaryKey(clientWeight.getSchoolId());
        if (clientWeight1 == null) {
            int insert = clientWeightMapper.insert(clientWeight);
            return insert == 1;
        } else {
            int i = clientWeightMapper.updateByPrimaryKey(clientWeight);
            return i == 1;
        }
    }


    //教师端 客户案例章节成绩详情
    @AnonymousAccess
    @ApiOperation("教师端-成绩中心-客户案例章节成绩详情")
    @PostMapping("selectTeaClientChapter")
    public ResultEntity<StuTrainingDto> selectTeaClientChapter(@RequestParam String userId,
                                                               @RequestParam String chapterName,
                                                               @RequestParam String schoolId) {
        ChapterExample chapterExample = new ChapterExample();
        chapterExample.createCriteria().andChapterNameEqualTo(chapterName);
        List<Chapter> chapters = chapterMapper.selectByExample(chapterExample);
        ResultEntity<StuTrainingDto> scoreInfo = getScoreInfo(userId, chapters.get(0).getChapterId(), schoolId);
        return scoreInfo;
    }


    //教师端 客户案例实验报告列表查询
    @AnonymousAccess
    @ApiOperation("教师端--客户案例实验报告列表查询")
    @PostMapping("selectTrainingReportList")
    public ResultEntity<PageInfo<TrainingReport>> selectTrainingReportList(@RequestParam Integer index,
                                                                           @RequestParam Integer size,
                                                                           @RequestParam String schoolId) {
        PageHelper.startPage(index, size);
        TrainingReportExample trainingReportExample = new TrainingReportExample();
        trainingReportExample.createCriteria().andSchoolIdEqualTo(schoolId);
        List<TrainingReport> trainingReports = trainingReportMapper.selectByExampleWithBLOBs(trainingReportExample);
        List<TrainingReportDto> list = new AstList();
        if (!trainingReports.isEmpty()) {
            for (TrainingReport trainingReport : trainingReports) {
                TrainingReportDto trainingReportDto = new TrainingReportDto();
                BeanUtils.copyProperties(trainingReport, trainingReportDto);
                UserInfo user = userInfoMapper.selectByPrimaryKey(trainingReport.getUserId());
                trainingReportDto.setName(user.getName());
                trainingReportDto.setStudentId(user.getStudentId());
                trainingReportDto.setClassName(user.getClassName());
                list.add(trainingReportDto);
            }
        }
        PageInfo pageInfo = new PageInfo(list);
        return new ResultEntity<PageInfo<TrainingReport>>(pageInfo);
    }


    //教师端 客户案例获取单个学生实训报告
    @GetMapping("/getReportByReportId")
    @ApiOperation("教师端--客户案例获取单个学生实训报告")
    @AnonymousAccess
    public ResponseEntity<FileSystemResource> getReport(@RequestParam String reportId, HttpServletResponse response) {
        TrainingReport trainingReport = trainingReportMapper.selectByPrimaryKey(reportId);
        String fileUrl = trainingReport.getFilePath();
        String videoPath = filePath + fileUrl;
        File videoFile = new File(videoPath);
        if (videoFile.exists()) {
            FileSystemResource fileSystemResource = new FileSystemResource(videoFile);
            return ResponseEntity.ok()
                    .contentType(MediaType.parseMediaType("application/pdf"))
                    .body(fileSystemResource);
        } else {
            return ResponseEntity.notFound().build();
        }
    }

    //教师端 客户案例评阅 学生信息列表查询  *****

    //教师端 客户案例实验报告评阅
    @AnonymousAccess
    @PostMapping("/updateScoreOrComment")
    @ApiOperation("教师端--实训报告评阅页面老师评分或评语")
    public ResultEntity updateWeightByModule(@RequestParam String reportId,
                                             @RequestParam(required = false) BigDecimal score,
                                             @RequestParam(required = false) String comment) {
        TrainingReport trainingReport = trainingReportMapper.selectByPrimaryKey(reportId);
        if (score == null || score.intValue() < 0 || score.intValue() > 100) {
            return new ResultEntity(HttpStatus.BAD_REQUEST, "请输入正确的分数！");
        } else {
            trainingReport.setTeacherScore(score);
        }
        if (StringUtils.isNotBlank(comment)) {
            trainingReport.setTeacherComment(comment);
        }
        trainingReportMapper.updateByPrimaryKey(trainingReport);
        return new ResultEntity(HttpStatus.OK, "修改成功！");
    }


    //教师端 综合规划列表查询
    @AnonymousAccess
    @ApiOperation("教师端-成绩中心-综合规划成绩展示")
    @PostMapping("selectTeaSynthesisPlanScoreList")
    public ResultEntity<PageInfo<ScoreRank>> selectTeaSynthesisPlanScoreList(@RequestParam String schoolId, @RequestParam Integer index, @RequestParam Integer size,
                                                                             @RequestParam(required = false) String name,
                                                                             @RequestParam(required = false) String studentId,
                                                                             @RequestParam(required = false) String classId) {
        PageHelper.startPage(index, size);
        ScoreRankExample scoreRankExample = new ScoreRankExample();
        ScoreRankExample.Criteria criteria = scoreRankExample.createCriteria();
        criteria.andSchoolIdEqualTo(schoolId).andUpdateDateEqualTo(scoreRankService.getNowDate());
        if (StringUtils.isNotBlank(name)) {
            criteria.andNameEqualTo(name);
        }
        if (StringUtils.isNotBlank(studentId)) {
            criteria.andSchoolIdEqualTo(studentId);
        }
        if (StringUtils.isNotBlank(classId)) {
            criteria.andClassIdEqualTo(classId);
        }
        scoreRankExample.setOrderByClause("zhgh_rank asc");
        List<ScoreRank> scoreRankList = scoreRankMapper.selectByExample(scoreRankExample);

        //设置总成绩
        for (int i = 0; i < scoreRankList.size(); i++) {
            ScoreRank scoreRank = scoreRankList.get(i);
            scoreRank.setTotalScore(scoreRank.getZhghScore());
        }
        PageInfo pageInfo = new PageInfo(scoreRankList);
        return new ResultEntity<PageInfo<ScoreRank>>(pageInfo);
    }

    //教师端 综合规划页面导出
    @AnonymousAccess
    @GetMapping("/exportSynthesisPlanScore")
    @ApiOperation("教师端--综合规划页面导出")
    public void exportSynthesisPlanScore(HttpServletResponse response,
                                         @RequestParam String schoolId) {
        ScoreRankExample scoreRankExample = new ScoreRankExample();
        ScoreRankExample.Criteria criteria = scoreRankExample.createCriteria();
        criteria.andSchoolIdEqualTo(schoolId).andUpdateDateEqualTo(scoreRankService.getNowDate());
        scoreRankExample.setOrderByClause("zhgh_score");
        List<ScoreRank> scoreRankList = scoreRankMapper.selectByExample(scoreRankExample);

        //导出的表名
        String title = IdUtil.simpleUUID();
        //表中第一行表头字段
        String[] headers = {"排名", "姓名", "学号", "班级", "总成绩", "完成案例个数"};
        //具体需要写入excel需要哪些字段，这些字段取自UserReward类，也就是上面的实际数据结果集的泛型
        List<String> listColumn = Arrays.asList("classId", "name", "studentId", "className", "zhghScore", "countCase");
        try {
            FilePortUtil.exportExcel(response, title, headers, scoreRankList, listColumn);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @GetMapping("/getSubmitStatusCaseListByUserId")
    @ApiOperation("教师端-成绩中心-综合规划-案例列表展示")
    @AnonymousAccess
    public ResultEntity<List<SynthesisPlanClient>> getSubmitStatusCaseListByUserId(String userId) {
        SynthesisPlanClientExample synthesisPlanClientExample = new SynthesisPlanClientExample();
        List<String> submitStatusList = new ArrayList<>();
        submitStatusList.add("未审核");
        submitStatusList.add("已审核");
        synthesisPlanClientExample.createCriteria().andUseridEqualTo(userId).andSubmitStatusIn(submitStatusList);
        List<SynthesisPlanClient> synthesisPlanClientList = synthesisPlanClientMapper.selectByExample(synthesisPlanClientExample);
        return new ResultEntity<>(HttpStatus.OK, "案例列表展示成功", synthesisPlanClientList);
    }

    //***** map有误 必须根据caseId获取案例题所有type
    @GetMapping("/getObjCaseScore")
    @ApiOperation("教师端-成绩中心-综合规划-查看详情、评分 客观题标准分、学生得分情况")
    @AnonymousAccess
    public ResultEntity<Map<String, ObjCaseScoreVo>> getObjCaseScore(String userId, String caseId) {
        SynthesisPlanScoreExample synthesisPlanScoreExample = new SynthesisPlanScoreExample();
        synthesisPlanScoreExample.createCriteria().andCaseidEqualTo(caseId).andUseridEqualTo(userId);
        List<SynthesisPlanScore> synthesisPlanScores = synthesisPlanScoreMapper.selectByExample(synthesisPlanScoreExample);
        if (!synthesisPlanScores.isEmpty()) {
            Map<String, ObjCaseScoreVo> map = new HashMap<>();
            for (int i = 0; i < synthesisPlanScores.size(); i++) {
                SynthesisPlanScore synthesisPlanScore = synthesisPlanScores.get(i);
                if(synthesisPlanScore.getTestCenter().contains("分析") || synthesisPlanScore.getTestCenter().contains("理财方案")){
                    continue;
                }
                if (map.containsKey(synthesisPlanScore.getManageMoneyMattersType())) {
                    ObjCaseScoreVo objCaseScoreVo = map.get(synthesisPlanScore.getManageMoneyMattersType());
                    objCaseScoreVo.setStandardScore(objCaseScoreVo.getStandardScore().add(BigDecimal.valueOf(2)));
                    if ("答对".equals(synthesisPlanScore.getStatus())) {
                        objCaseScoreVo.setStuScore(objCaseScoreVo.getStuScore().add(BigDecimal.valueOf(2)));
                    }
                } else {
                    ObjCaseScoreVo objCaseScoreVo = new ObjCaseScoreVo();
                    objCaseScoreVo.setStandardScore(BigDecimal.valueOf(2));
                    if ("答对".equals(synthesisPlanScore.getStatus())) {
                        objCaseScoreVo.setStuScore(BigDecimal.valueOf(2));
                    } else {
                        objCaseScoreVo.setStuScore(BigDecimal.ZERO);
                    }
                    map.put(synthesisPlanScore.getManageMoneyMattersType(), objCaseScoreVo);
                }
            }

            Set<String> keySet = map.keySet();
            for (Object key : keySet) {
                ObjCaseScoreVo objCaseScoreVo = map.get(key);
                if ("财务分析-财务比例分析".equals(key)) {
                    CaseFinancialRatiosExample example = new CaseFinancialRatiosExample();
                    example.createCriteria().andCaseidEqualTo(caseId).andIsAnswerEqualTo("题目答案");
                    List<CaseFinancialRatiosWithBLOBs> caseFinancialRatios = caseFinancialRatiosMapper.selectByExampleWithBLOBs(example);
                    if (!caseFinancialRatios.isEmpty()) {
                        objCaseScoreVo.setAnswerOne(caseFinancialRatios.get(0).getAnalysis());
                    }
                }
                if ("现金规划".equals(key)) {
                    CaseCashPlanExample example = new CaseCashPlanExample();
                    example.createCriteria().andCaseidEqualTo(caseId).andIsAnswerEqualTo("题目答案");
                    List<CaseCashPlan> teaObj = caseCashPlanMapper.selectByExampleWithBLOBs(example);
                    if (!teaObj.isEmpty()) {
                        objCaseScoreVo.setAnswerOne(teaObj.get(0).getAnalysis());
                    }
                }
                if ("生涯规划-教育规划".equals(key)) {
                    CaseLifeEducationPlanExample example = new CaseLifeEducationPlanExample();
                    example.createCriteria().andCaseidEqualTo(caseId).andIsAnswerEqualTo("题目答案");
                    List<CaseLifeEducationPlan> teaObj = caseLifeEducationPlanMapper.selectByExampleWithBLOBs(example);
                    if (!teaObj.isEmpty()) {
                        objCaseScoreVo.setAnswerOne(String.valueOf(teaObj.get(0).getFinancialPlan()));
                        objCaseScoreVo.setAnswerTwo(teaObj.get(0).getAnalysis());
                    }
                }
                if ("生涯规划-消费规划".equals(key)) {
                    CaseConsumptionPlanExample example = new CaseConsumptionPlanExample();
                    example.createCriteria().andCaseIdEqualTo(caseId).andIsAnswerEqualTo("题目答案");
                    List<CaseConsumptionPlan> teaObj = caseConsumptionPlanMapper.selectByExampleWithBLOBs(example);
                    if (!teaObj.isEmpty()) {
                        objCaseScoreVo.setAnswerOne(String.valueOf(teaObj.get(0).getFinancialPlan()));
                        objCaseScoreVo.setAnswerTwo(teaObj.get(0).getAnalysis());
                    }
                }
                if ("生涯规划-创业规划".equals(key)) {
                    CaseStartAnUndertakingPlanExample example = new CaseStartAnUndertakingPlanExample();
                    example.createCriteria().andCaseidEqualTo(caseId).andIsAnswerEqualTo("题目答案");
                    List<CaseStartAnUndertakingPlan> teaObj = caseStartAnUndertakingPlanMapper.selectByExample(example);
                    if (!teaObj.isEmpty()) {
                        objCaseScoreVo.setAnswerOne(String.valueOf(teaObj.get(0).getFinancialPlan()));
                        objCaseScoreVo.setAnswerTwo(teaObj.get(0).getAnalysis());
                    }
                }
                if ("生涯规划-退休规划".equals(key)) {
                    CaseRetirementPlanExample example = new CaseRetirementPlanExample();
                    example.createCriteria().andCaseidEqualTo(caseId).andIsAnswerEqualTo("题目答案");
                    List<CaseRetirementPlan> teaObj = caseRetirementPlanMapper.selectByExample(example);
                    if (!teaObj.isEmpty()) {
                        objCaseScoreVo.setAnswerOne(String.valueOf(teaObj.get(0).getFinancialPlan()));
                        objCaseScoreVo.setAnswerTwo(teaObj.get(0).getAnalysis());
                    }
                }
                if ("生涯规划-保险规划".equals(key)) {
                    CaseInsurancePlanExample example = new CaseInsurancePlanExample();
                    example.createCriteria().andCaseidEqualTo(caseId).andIsAnswerEqualTo("题目答案");
                    List<CaseInsurancePlan> teaObj = caseInsurancePlanMapper.selectByExample(example);
                    if (!teaObj.isEmpty()) {
                        objCaseScoreVo.setAnswerOne(teaObj.get(0).getAnalysis());
                    }
                }
                if ("投资规划".equals(key)) {
                    CaseInvestmentPlanExample example = new CaseInvestmentPlanExample();
                    example.createCriteria().andCaseIdEqualTo(caseId).andIsAnswerEqualTo("题目答案");
                    List<CaseInvestmentPlan> teaObj = caseInvestmentPlanMapper.selectByExample(example);
                    if (!teaObj.isEmpty()) {
                        objCaseScoreVo.setAnswerOne(teaObj.get(0).getAnalysis());
                    }
                }
                if ("税务筹划".equals(key)) {
                    CaseTaxPlanExample example = new CaseTaxPlanExample();
                    example.createCriteria().andCaseidEqualTo(caseId).andIsAnswerEqualTo("题目答案");
                    List<CaseTaxPlan> teaObj = caseTaxPlanMapper.selectByExample(example);
                    if (!teaObj.isEmpty()) {
                        objCaseScoreVo.setAnswerOne(teaObj.get(0).getAnalysis());
                    }
                }
                if ("财产分配".equals(key)) {
                    CaseDistributionOfPropertyExample example = new CaseDistributionOfPropertyExample();
                    example.createCriteria().andCaseidEqualTo(caseId).andIsAnswerEqualTo("题目答案");
                    List<CaseDistributionOfPropertyWithBLOBs> teaObj = caseDistributionOfPropertyMapper.selectByExampleWithBLOBs(example);
                    if (!teaObj.isEmpty()) {
                        objCaseScoreVo.setAnswerOne(String.valueOf(teaObj.get(0).getSituationAnalysis()));
                        objCaseScoreVo.setAnswerTwo(teaObj.get(0).getPlanAnalysis());
                    }
                }
                if ("财产传承".equals(key)) {
                    CaseHeritageExample example = new CaseHeritageExample();
                    example.createCriteria().andCaseidEqualTo(caseId).andIsAnswerEqualTo("题目答案");
                    List<CaseHeritageWithBLOBs> teaObj = caseHeritageMapper.selectByExampleWithBLOBs(example);
                    if (!teaObj.isEmpty()) {
                        objCaseScoreVo.setAnswerOne(String.valueOf(teaObj.get(0).getFinanceAnalysis()));
                        objCaseScoreVo.setAnswerTwo(teaObj.get(0).getPlanAnalysis());
                    }
                }

            }
            return new ResultEntity<>(HttpStatus.OK, "学生得分情况返回成功", map);
        } else {
            return new ResultEntity<>(HttpStatus.OK, "未查询到数据");
        }
    }

    //教师端 综合规划成绩详情查询
//    @PostMapping("/getExamPoint")
//    @ApiOperation("教师端 综合规划成绩详情查询")
//    @AnonymousAccess
//    private ResultEntity<List<SynthesisPlanScoreWithBLOBs>> getExamPoint(@RequestParam String caseID,@RequestParam String userId) {
//        SynthesisPlanScoreExample synthesisPlanScoreExample = new SynthesisPlanScoreExample();
//        synthesisPlanScoreExample.createCriteria().andUseridEqualTo(userId).andCaseidEqualTo(caseID);
//        List<SynthesisPlanScoreWithBLOBs> synthesisPlanScores = synthesisPlanScoreMapper.selectByExampleWithBLOBs(synthesisPlanScoreExample);
//        return new ResultEntity<>(HttpStatus.OK, "综合规划成绩详情查询成功",synthesisPlanScores);
//    }

    //教师端 综合规划老师评分
    @PostMapping("/updateSynthesisPlanMark")
    @ApiOperation("教师端 综合规划老师评分")
    @AnonymousAccess
    private ResultEntity updateSynthesisPlanMark(@ApiParam("map<type,json> type为更新的表名，json为预览展示单个表中的对象") @RequestBody Map<String, Object> updateMap,
                                                 @RequestParam String userId, @RequestParam String caseId) {
        System.out.println();
        for (String type : updateMap.keySet()) {
//            String json = updateMap.get(type);
            LinkedHashMap<String, String> map = (LinkedHashMap<String, String>) updateMap.get(type);
            String json = new Gson().toJson(map);
            Gson gson = new Gson();
            caseService.addCorrelationTable(type, json, gson, "1");
        }
        //修改审核状态
        SynthesisPlanClientExample synthesisPlanClientExample = new SynthesisPlanClientExample();
        synthesisPlanClientExample.createCriteria().andUseridEqualTo(userId).andCaseidEqualTo(caseId);
        List<SynthesisPlanClient> synthesisPlanClients = synthesisPlanClientMapper.selectByExample(synthesisPlanClientExample);
        SynthesisPlanClient synthesisPlanClient = synthesisPlanClients.get(0);
        synthesisPlanClient.setSubmitStatus("已审核");
        synthesisPlanClientMapper.updateByPrimaryKeySelective(synthesisPlanClient);
        //设置综合规划成绩表 *****

        return new ResultEntity<>(HttpStatus.OK, "更新评分成功");
    }

    //教师端 综合规划一键评分
    @GetMapping("/OneClickScoring")
    @ApiOperation("教师端-综合规划一键评分")
    @AnonymousAccess
    private ResultEntity OneClickScoring(String schoolId) {
        //先在synthesis_plan_client表中查询schoolID  并且submitStatus状态为未审核的数据 循环获取caseId和userId
        SynthesisPlanClientExample synthesisPlanClientExample = new SynthesisPlanClientExample();
        synthesisPlanClientExample.createCriteria().andSchoolIdEqualTo(schoolId).andSubmitStatusEqualTo("未审核");
        List<SynthesisPlanClient> planClientList = synthesisPlanClientMapper.selectByExample(synthesisPlanClientExample);

        for (int i = 0; i < planClientList.size(); i++) {
            SynthesisPlanClient client = planClientList.get(0);
            String caseid = client.getCaseid();
            String userid = client.getUserid();
            //根据caseId 查询caseInfo表中的manage_money_matters_type字段 如果等于综合规划 则查询com_case表 获取理财类型TypeList 如果不等于综合规划 则获取caseInfo中的manage_money_matters_type作为理财类型Type
            CaseInfo caseInfo = caseInfoMapper.selectByPrimaryKey(caseid);
            List<String> typeList = new ArrayList<>();
            if (caseInfo.getManageMoneyMattersType().contains(",")) {
                String[] split = caseInfo.getManageMoneyMattersType().split(",");
                for (String type : split) {
                    typeList.add(type);
                }
            } else {
                typeList.add(caseInfo.getManageMoneyMattersType());
            }
            for (int j = 0; j < typeList.size(); j++) {
                String type = typeList.get(j);
                if ("财务分析-财务比例分析".equals(type)) {
                    //根据caseid和userId查询case_Financial_Ratios表
                    CaseFinancialRatiosExample example = new CaseFinancialRatiosExample();
                    example.createCriteria().andCaseidEqualTo(caseid).andUserIdEqualTo(userid);
                    List<CaseFinancialRatiosWithBLOBs> list = caseFinancialRatiosMapper.selectByExampleWithBLOBs(example);
                    //如果不存在 则跳过
                    if (list.isEmpty()) {
                        continue;
                    }
                    CaseFinancialRatiosWithBLOBs caseFinancialRatiosWithBLOBs = list.get(0);
                    String analysis = caseFinancialRatiosWithBLOBs.getAnalysis();
                    BigDecimal teacherScore = BigDecimal.ZERO;
                    if (StringUtils.isBlank(analysis)) {
                        caseFinancialRatiosWithBLOBs.setAnalysisTeacherScore(teacherScore);
                    } else {
                        if (analysis.length() < 20) {
                            caseFinancialRatiosWithBLOBs.setAnalysisTeacherScore(teacherScore);
                        } else if (analysis.length() >= 50) {
                            teacherScore = BigDecimal.valueOf(2);
                            caseFinancialRatiosWithBLOBs.setAnalysisTeacherScore(teacherScore);
                        } else {
                            teacherScore = BigDecimal.valueOf(1);
                            caseFinancialRatiosWithBLOBs.setAnalysisTeacherScore(teacherScore);
                        }
                    }
                    caseFinancialRatiosMapper.updateByPrimaryKeyWithBLOBs(caseFinancialRatiosWithBLOBs);
                    //并设置综合规划成绩表 修改分数
                    SynthesisPlanScoreExample scoreExample = new SynthesisPlanScoreExample();
                    scoreExample.createCriteria().andCaseidEqualTo(caseid).andUseridEqualTo(userid).andManageMoneyMattersTypeEqualTo("财务分析-财务比例分析").andTestCenterEqualTo("客户财务情况分析");
                    List<SynthesisPlanScore> scoreList = synthesisPlanScoreMapper.selectByExample(scoreExample);
                    SynthesisPlanScore synthesisPlanScore = scoreList.get(0);
                    synthesisPlanScore.setScore(teacherScore);
                    if(teacherScore.compareTo(BigDecimal.ZERO)>0){
                        synthesisPlanScore.setStatus("答对");
                    }else {
                        synthesisPlanScore.setStatus("答错");
                    }
                    synthesisPlanScoreMapper.updateByPrimaryKey(synthesisPlanScore);
                } else if ("现金规划".equals(type)) {
                    CaseCashPlanExample example = new CaseCashPlanExample();
                    example.createCriteria().andCaseidEqualTo(caseid).andUserIdEqualTo(userid);
                    List<CaseCashPlan> list = caseCashPlanMapper.selectByExample(example);
                    //如果不存在 则跳过
                    if (list.isEmpty()) {
                        continue;
                    }
                    CaseCashPlan caseCashPlan = list.get(0);
                    String analysis = caseCashPlan.getAnalysis();
                    BigDecimal teacherScore = BigDecimal.ZERO;
                    if (StringUtils.isBlank(analysis)) {
                        caseCashPlan.setAnalysisTeacherScore(teacherScore);
                    } else {
                        if (analysis.length() < 20) {
                            caseCashPlan.setAnalysisTeacherScore(teacherScore);
                        } else if (analysis.length() >= 50) {
                            teacherScore = BigDecimal.valueOf(2);
                            caseCashPlan.setAnalysisTeacherScore(teacherScore);
                        } else {
                            teacherScore = BigDecimal.valueOf(1);
                            caseCashPlan.setAnalysisTeacherScore(teacherScore);
                        }
                    }
                    caseCashPlanMapper.updateByPrimaryKey(caseCashPlan);
                    //并设置综合规划成绩表 修改分数
                    SynthesisPlanScoreExample scoreExample = new SynthesisPlanScoreExample();
                    scoreExample.createCriteria().andCaseidEqualTo(caseid).andUseridEqualTo(userid).andManageMoneyMattersTypeEqualTo("现金规划").andTestCenterEqualTo("现金规划分析");
                    List<SynthesisPlanScore> scoreList = synthesisPlanScoreMapper.selectByExample(scoreExample);
                    SynthesisPlanScore synthesisPlanScore = scoreList.get(0);
                    synthesisPlanScore.setScore(teacherScore);
                    if(teacherScore.compareTo(BigDecimal.ZERO)>0){
                        synthesisPlanScore.setStatus("答对");
                    }else {
                        synthesisPlanScore.setStatus("答错");
                    }
                    synthesisPlanScoreMapper.updateByPrimaryKey(synthesisPlanScore);
                } else if ("生涯规划-教育规划".equals(type)) {
                    CaseLifeEducationPlanExample example = new CaseLifeEducationPlanExample();
                    example.createCriteria().andCaseidEqualTo(caseid).andUserIdEqualTo(userid);
                    List<CaseLifeEducationPlan> list = caseLifeEducationPlanMapper.selectByExample(example);
                    //如果不存在 则跳过
                    if (list.isEmpty()) {
                        continue;
                    }
                    CaseLifeEducationPlan stuPlan = list.get(0);
                    String analysis = stuPlan.getAnalysis();
                    BigDecimal teacherScore = BigDecimal.ZERO;
                    //设置分析分数
                    if (StringUtils.isBlank(analysis)) {
                        stuPlan.setAnalysisTeacherScore(teacherScore);
                    } else {
                        if (analysis.length() < 20) {
                            stuPlan.setAnalysisTeacherScore(teacherScore);
                        } else if (analysis.length() >= 50) {
                            teacherScore = BigDecimal.valueOf(2);
                            stuPlan.setAnalysisTeacherScore(teacherScore);
                        } else {
                            teacherScore = BigDecimal.valueOf(1);
                            stuPlan.setAnalysisTeacherScore(teacherScore);
                        }
                    }
                    //设置理财方案分数 先获取案例题答案的分数 再获取学生理财方案的四个对象 按照公式对比大小 如果大于老师的答案 则给两分 低于给0分
                    BigDecimal teaFinancialPlanningMoney = new BigDecimal(0);
                    String financialPlan = stuPlan.getFinancialPlan();
                    try {
                        if (financialPlan != null) {
                            teaFinancialPlanningMoney = new BigDecimal(financialPlan);
                        }
                    } catch (Exception e) {
                        // 可以选择设置一个默认值或者进行其他错误处理
                        teaFinancialPlanningMoney = new BigDecimal("0"); // 或者其他适当的默认值
                    }
                    BigDecimal returnOnInvestment = stuPlan.getReturnOnInvestment();
                    BigDecimal disposableInput = stuPlan.getDisposableInput();
                    BigDecimal monthlyInvestment = stuPlan.getMonthlyInvestment();
                    Integer regularYear = stuPlan.getRegularYear();  //定期定额投资年限 n
                    BigDecimal teacherFinancialPlanningScore = BigDecimal.ZERO;  //老师理财分数 用于保存到综合规划成绩表中
                    try {
                        BigDecimal stuFinancialPlanningMoney = getFinancialPlanningMoney(returnOnInvestment, disposableInput, monthlyInvestment, regularYear);
                        if (stuFinancialPlanningMoney.compareTo(teaFinancialPlanningMoney) >= 0) {
                            teacherFinancialPlanningScore = BigDecimal.valueOf(2);
                            stuPlan.setCaseTeacherScore(teacherFinancialPlanningScore);
                        } else {
                            stuPlan.setCaseTeacherScore(teacherFinancialPlanningScore);
                        }
                    } catch (Exception e) {

                    }
                    caseLifeEducationPlanMapper.updateByPrimaryKey(stuPlan);
                    //并设置综合规划成绩表 修改老师评分分数
                    SynthesisPlanScoreExample scoreExample = new SynthesisPlanScoreExample();
                    scoreExample.createCriteria().andCaseidEqualTo(caseid).andUseridEqualTo(userid).andManageMoneyMattersTypeEqualTo("生涯规划-教育规划").andTestCenterEqualTo("教育规划分析");
                    List<SynthesisPlanScore> scoreList = synthesisPlanScoreMapper.selectByExample(scoreExample);
                    SynthesisPlanScore synthesisPlanScore = scoreList.get(0);
                    synthesisPlanScore.setScore(teacherScore);
                    if(teacherScore.compareTo(BigDecimal.ZERO)>0){
                        synthesisPlanScore.setStatus("答对");
                    }else {
                        synthesisPlanScore.setStatus("答错");
                    }
                    synthesisPlanScoreMapper.updateByPrimaryKey(synthesisPlanScore);

                    //修改理财方案老师评分
                    SynthesisPlanScoreExample scoreExample2 = new SynthesisPlanScoreExample();
                    scoreExample2.createCriteria().andCaseidEqualTo(caseid).andUseridEqualTo(userid).andManageMoneyMattersTypeEqualTo("生涯规划-教育规划").andTestCenterEqualTo("理财方案");
                    List<SynthesisPlanScore> scoreList2 = synthesisPlanScoreMapper.selectByExample(scoreExample2);
                    SynthesisPlanScore synthesisPlanScore2 = scoreList2.get(0);
                    synthesisPlanScore2.setScore(teacherFinancialPlanningScore);
                    if(teacherFinancialPlanningScore.compareTo(BigDecimal.ZERO)>0){
                        synthesisPlanScore2.setStatus("答对");
                    }else {
                        synthesisPlanScore2.setStatus("答错");
                    }
                    synthesisPlanScoreMapper.updateByPrimaryKey(synthesisPlanScore2);
                } else if ("生涯规划-消费规划".equals(type)) {
                    CaseConsumptionPlanExample example = new CaseConsumptionPlanExample();
                    example.createCriteria().andCaseIdEqualTo(caseid).andUserIdEqualTo(userid);
                    List<CaseConsumptionPlan> list = caseConsumptionPlanMapper.selectByExample(example);
                    //如果不存在 则跳过
                    if (list.isEmpty()) {
                        continue;
                    }
                    CaseConsumptionPlan stuPlan = list.get(0);
                    String analysis = stuPlan.getAnalysis();
                    BigDecimal teacherScore = BigDecimal.ZERO;
                    //设置分析分数
                    if (StringUtils.isBlank(analysis)) {
                        stuPlan.setAnalysisTeacherScore(teacherScore);
                    } else {
                        if (analysis.length() < 20) {
                            stuPlan.setAnalysisTeacherScore(teacherScore);
                        } else if (analysis.length() >= 50) {
                            teacherScore = BigDecimal.valueOf(2);
                            stuPlan.setAnalysisTeacherScore(teacherScore);
                        } else {
                            teacherScore = BigDecimal.valueOf(1);
                            stuPlan.setAnalysisTeacherScore(teacherScore);
                        }
                    }
                    //设置理财方案分数 先获取案例题答案的分数 再获取学生理财方案的四个对象 按照公式对比大小 如果大于老师的答案 则给两分 低于给0分
                    BigDecimal teaFinancialPlanningMoney = new BigDecimal(0);
                    String financialPlan = stuPlan.getFinancialPlan();
                    try {
                        if (financialPlan != null) {
                            teaFinancialPlanningMoney = new BigDecimal(financialPlan);
                        }
                    } catch (Exception e) {
                        // 可以选择设置一个默认值或者进行其他错误处理
                        teaFinancialPlanningMoney = new BigDecimal("0"); // 或者其他适当的默认值
                    }
                    BigDecimal returnOnInvestment = stuPlan.getReturnOnInvestment();
                    BigDecimal disposableInput = stuPlan.getDisposableInput();
                    BigDecimal monthlyInvestment = stuPlan.getMonthlyInvestment();
                    Integer regularYear = stuPlan.getRegularYear();  //定期定额投资年限 n
                    BigDecimal teacherFinancialPlanningScore = BigDecimal.ZERO;  //老师理财分数 用于保存到综合规划成绩表中
                    try {
                        BigDecimal stuFinancialPlanningMoney = getFinancialPlanningMoney(returnOnInvestment, disposableInput, monthlyInvestment, regularYear);
                        if (stuFinancialPlanningMoney.compareTo(teaFinancialPlanningMoney) >= 0) {
                            teacherFinancialPlanningScore = BigDecimal.valueOf(2);
                            stuPlan.setCaseTeacherScore(teacherFinancialPlanningScore);
                        } else {
                            stuPlan.setCaseTeacherScore(teacherFinancialPlanningScore);
                        }
                    } catch (Exception e) {

                    }
                    caseConsumptionPlanMapper.updateByPrimaryKey(stuPlan);
                    //并设置综合规划成绩表 修改老师评分分数
                    SynthesisPlanScoreExample scoreExample = new SynthesisPlanScoreExample();
                    scoreExample.createCriteria().andCaseidEqualTo(caseid).andUseridEqualTo(userid).andManageMoneyMattersTypeEqualTo("生涯规划-消费规划").andTestCenterEqualTo("消费规划分析");
                    List<SynthesisPlanScore> scoreList = synthesisPlanScoreMapper.selectByExample(scoreExample);
                    SynthesisPlanScore synthesisPlanScore = scoreList.get(0);
                    synthesisPlanScore.setScore(teacherScore);
                    if(teacherScore.compareTo(BigDecimal.ZERO)>0){
                        synthesisPlanScore.setStatus("答对");
                    }else {
                        synthesisPlanScore.setStatus("答错");
                    }
                    synthesisPlanScoreMapper.updateByPrimaryKey(synthesisPlanScore);

                    //修改理财方案老师评分
                    SynthesisPlanScoreExample scoreExample2 = new SynthesisPlanScoreExample();
                    scoreExample2.createCriteria().andCaseidEqualTo(caseid).andUseridEqualTo(userid).andManageMoneyMattersTypeEqualTo("生涯规划-消费规划").andTestCenterEqualTo("理财方案");
                    List<SynthesisPlanScore> scoreList2 = synthesisPlanScoreMapper.selectByExample(scoreExample2);
                    SynthesisPlanScore synthesisPlanScore2 = scoreList2.get(0);
                    synthesisPlanScore2.setScore(teacherFinancialPlanningScore);
                    if(teacherFinancialPlanningScore.compareTo(BigDecimal.ZERO)>0){
                        synthesisPlanScore2.setStatus("答对");
                    }else {
                        synthesisPlanScore2.setStatus("答错");
                    }
                    synthesisPlanScoreMapper.updateByPrimaryKey(synthesisPlanScore2);
                } else if ("生涯规划-创业规划".equals(type)) {
                    CaseStartAnUndertakingPlanExample example = new CaseStartAnUndertakingPlanExample();
                    example.createCriteria().andCaseidEqualTo(caseid).andUserIdEqualTo(userid);
                    List<CaseStartAnUndertakingPlan> list = caseStartAnUndertakingPlanMapper.selectByExample(example);
                    //如果不存在 则跳过
                    if (list.isEmpty()) {
                        continue;
                    }
                    CaseStartAnUndertakingPlan stuPlan = list.get(0);
                    String analysis = stuPlan.getAnalysis();
                    BigDecimal teacherScore = BigDecimal.ZERO;
                    //设置分析分数
                    if (StringUtils.isBlank(analysis)) {
                        stuPlan.setAnalysisTeacherScore(teacherScore);
                    } else {
                        if (analysis.length() < 20) {
                            stuPlan.setAnalysisTeacherScore(teacherScore);
                        } else if (analysis.length() >= 50) {
                            teacherScore = BigDecimal.valueOf(2);
                            stuPlan.setAnalysisTeacherScore(teacherScore);
                        } else {
                            teacherScore = BigDecimal.valueOf(1);
                            stuPlan.setAnalysisTeacherScore(teacherScore);
                        }
                    }
                    //设置理财方案分数 先获取案例题答案的分数 再获取学生理财方案的四个对象 按照公式对比大小 如果大于老师的答案 则给两分 低于给0分
                    BigDecimal teaFinancialPlanningMoney = new BigDecimal(0);
                    String financialPlan = stuPlan.getFinancialPlan();
                    try {
                        if (financialPlan != null) {
                            teaFinancialPlanningMoney = new BigDecimal(financialPlan);
                        }
                    } catch (Exception e) {
                        // 可以选择设置一个默认值或者进行其他错误处理
                        teaFinancialPlanningMoney = new BigDecimal("0"); // 或者其他适当的默认值
                    }
                    BigDecimal returnOnInvestment = stuPlan.getReturnOnInvestmentRate();
                    BigDecimal disposableInput = stuPlan.getDisposableInput();
                    BigDecimal monthlyInvestment = stuPlan.getMonthlyInvestment();
                    Integer regularYear = stuPlan.getRegularYear();  //定期定额投资年限 n
                    BigDecimal teacherFinancialPlanningScore = BigDecimal.ZERO;  //老师理财分数 用于保存到综合规划成绩表中
                    try {
                        BigDecimal stuFinancialPlanningMoney = getFinancialPlanningMoney(returnOnInvestment, disposableInput, monthlyInvestment, regularYear);
                        if (stuFinancialPlanningMoney.compareTo(teaFinancialPlanningMoney) >= 0) {
                            teacherFinancialPlanningScore = BigDecimal.valueOf(2);
                            stuPlan.setCaseTeacherScore(teacherFinancialPlanningScore);
                        } else {
                            stuPlan.setCaseTeacherScore(teacherFinancialPlanningScore);
                        }
                    } catch (Exception e) {

                    }


                    caseStartAnUndertakingPlanMapper.updateByPrimaryKey(stuPlan);
                    //并设置综合规划成绩表 修改老师评分分数
                    SynthesisPlanScoreExample scoreExample = new SynthesisPlanScoreExample();
                    scoreExample.createCriteria().andCaseidEqualTo(caseid).andUseridEqualTo(userid).andManageMoneyMattersTypeEqualTo("生涯规划-创业规划").andTestCenterEqualTo("创业规划分析");
                    List<SynthesisPlanScore> scoreList = synthesisPlanScoreMapper.selectByExample(scoreExample);
                    SynthesisPlanScore synthesisPlanScore = scoreList.get(0);
                    synthesisPlanScore.setScore(teacherScore);
                    if(teacherScore.compareTo(BigDecimal.ZERO)>0){
                        synthesisPlanScore.setStatus("答对");
                    }else {
                        synthesisPlanScore.setStatus("答错");
                    }
                    synthesisPlanScoreMapper.updateByPrimaryKey(synthesisPlanScore);

                    //修改理财方案老师评分
                    SynthesisPlanScoreExample scoreExample2 = new SynthesisPlanScoreExample();
                    scoreExample2.createCriteria().andCaseidEqualTo(caseid).andUseridEqualTo(userid).andManageMoneyMattersTypeEqualTo("生涯规划-创业规划").andTestCenterEqualTo("理财方案");
                    List<SynthesisPlanScore> scoreList2 = synthesisPlanScoreMapper.selectByExample(scoreExample2);
                    SynthesisPlanScore synthesisPlanScore2 = scoreList2.get(0);
                    synthesisPlanScore2.setScore(teacherFinancialPlanningScore);
                    if(teacherFinancialPlanningScore.compareTo(BigDecimal.ZERO)>0){
                        synthesisPlanScore2.setStatus("答对");
                    }else {
                        synthesisPlanScore2.setStatus("答错");
                    }
                    synthesisPlanScoreMapper.updateByPrimaryKey(synthesisPlanScore2);
                } else if ("生涯规划-退休规划".equals(type)) {
                    CaseRetirementPlanExample example = new CaseRetirementPlanExample();
                    example.createCriteria().andCaseidEqualTo(caseid).andUserIdEqualTo(userid);
                    List<CaseRetirementPlan> list = caseRetirementPlanMapper.selectByExample(example);
                    //如果不存在 则跳过
                    if (list.isEmpty()) {
                        continue;
                    }
                    CaseRetirementPlan stuPlan = list.get(0);
                    String analysis = stuPlan.getAnalysis();
                    BigDecimal teacherScore = BigDecimal.ZERO;
                    //设置分析分数
                    if (StringUtils.isBlank(analysis)) {
                        stuPlan.setAnalysisTeacherScore(teacherScore);
                    } else {
                        if (analysis.length() < 20) {
                            stuPlan.setAnalysisTeacherScore(teacherScore);
                        } else if (analysis.length() >= 50) {
                            teacherScore = BigDecimal.valueOf(2);
                            stuPlan.setAnalysisTeacherScore(teacherScore);
                        } else {
                            teacherScore = BigDecimal.valueOf(1);
                            stuPlan.setAnalysisTeacherScore(teacherScore);
                        }
                    }
                    //设置理财方案分数 先获取案例题答案的分数 再获取学生理财方案的四个对象 按照公式对比大小 如果大于老师的答案 则给两分 低于给0分
                    BigDecimal teaFinancialPlanningMoney = new BigDecimal(0);
                    String financialPlan = stuPlan.getFinancialPlan();
                    try {
                        if (financialPlan != null) {
                            teaFinancialPlanningMoney = new BigDecimal(financialPlan);
                        }
                    } catch (Exception e) {
                        // 可以选择设置一个默认值或者进行其他错误处理
                        teaFinancialPlanningMoney = new BigDecimal("0"); // 或者其他适当的默认值
                    }
                    BigDecimal returnOnInvestment = stuPlan.getReturnOnInvestmentRate();
                    BigDecimal disposableInput = stuPlan.getDisposableInput();
                    BigDecimal monthlyInvestment = stuPlan.getMonthlyInvestment();
                    Integer regularYear = stuPlan.getRegularYear();  //定期定额投资年限 n
                    BigDecimal teacherFinancialPlanningScore = BigDecimal.ZERO;  //老师理财分数 用于保存到综合规划成绩表中
                    try {
                        BigDecimal stuFinancialPlanningMoney = getFinancialPlanningMoney(returnOnInvestment, disposableInput, monthlyInvestment, regularYear);
                        if (stuFinancialPlanningMoney.compareTo(teaFinancialPlanningMoney) >= 0) {
                            teacherFinancialPlanningScore = BigDecimal.valueOf(2);
                            stuPlan.setCaseTeacherScore(teacherFinancialPlanningScore);
                        } else {
                            stuPlan.setCaseTeacherScore(teacherFinancialPlanningScore);
                        }
                    } catch (Exception e) {

                    }

                    caseRetirementPlanMapper.updateByPrimaryKey(stuPlan);
                    //并设置综合规划成绩表 修改老师评分分数
                    SynthesisPlanScoreExample scoreExample = new SynthesisPlanScoreExample();
                    scoreExample.createCriteria().andCaseidEqualTo(caseid).andUseridEqualTo(userid).andManageMoneyMattersTypeEqualTo("生涯规划-退休规划").andTestCenterEqualTo("退休规划分析");
                    List<SynthesisPlanScore> scoreList = synthesisPlanScoreMapper.selectByExample(scoreExample);
                    SynthesisPlanScore synthesisPlanScore = scoreList.get(0);
                    synthesisPlanScore.setScore(teacherScore);
                    if(teacherScore.compareTo(BigDecimal.ZERO)>0){
                        synthesisPlanScore.setStatus("答对");
                    }else {
                        synthesisPlanScore.setStatus("答错");
                    }
                    synthesisPlanScoreMapper.updateByPrimaryKey(synthesisPlanScore);

                    //修改理财方案老师评分
                    SynthesisPlanScoreExample scoreExample2 = new SynthesisPlanScoreExample();
                    scoreExample2.createCriteria().andCaseidEqualTo(caseid).andUseridEqualTo(userid).andManageMoneyMattersTypeEqualTo("生涯规划-退休规划").andTestCenterEqualTo("理财方案");
                    List<SynthesisPlanScore> scoreList2 = synthesisPlanScoreMapper.selectByExample(scoreExample2);
                    SynthesisPlanScore synthesisPlanScore2 = scoreList2.get(0);
                    synthesisPlanScore2.setScore(teacherFinancialPlanningScore);
                    if(teacherFinancialPlanningScore.compareTo(BigDecimal.ZERO)>0){
                        synthesisPlanScore2.setStatus("答对");
                    }else {
                        synthesisPlanScore2.setStatus("答错");
                    }
                    synthesisPlanScoreMapper.updateByPrimaryKey(synthesisPlanScore2);
                } else if ("生涯规划-保险规划".equals(type)) {
                    //根据caseid和userId查询case_Financial_Ratios表
                    CaseInsurancePlanExample example = new CaseInsurancePlanExample();
                    example.createCriteria().andCaseidEqualTo(caseid).andUserIdEqualTo(userid);
                    List<CaseInsurancePlan> list = caseInsurancePlanMapper.selectByExample(example);
                    //如果不存在 则跳过
                    if (list.isEmpty()) {
                        continue;
                    }
                    CaseInsurancePlan stuPlan = list.get(0);
                    String analysis = stuPlan.getAnalysis();
                    BigDecimal teacherScore = BigDecimal.ZERO;
                    if (StringUtils.isBlank(analysis)) {
                        stuPlan.setAnalysisTeacherScore(teacherScore);
                    } else {
                        if (analysis.length() < 20) {
                            stuPlan.setAnalysisTeacherScore(teacherScore);
                        } else if (analysis.length() >= 50) {
                            teacherScore = BigDecimal.valueOf(2);
                            stuPlan.setAnalysisTeacherScore(teacherScore);
                        } else {
                            teacherScore = BigDecimal.valueOf(1);
                            stuPlan.setAnalysisTeacherScore(teacherScore);
                        }
                    }
                    caseInsurancePlanMapper.updateByPrimaryKey(stuPlan);
                    //并设置综合规划成绩表 修改分数
                    SynthesisPlanScoreExample scoreExample = new SynthesisPlanScoreExample();
                    List<String> testCenterList = new ArrayList<>();
                    testCenterList.add("寿险需求测算方法选择-遗属需求法-保险规划分析");
                    testCenterList.add("寿险需求测算方法选择-生命价值法-保险规划分析");
                    scoreExample.createCriteria().andCaseidEqualTo(caseid).andUseridEqualTo(userid).andManageMoneyMattersTypeEqualTo("生涯规划-保险规划").andTestCenterIn(testCenterList);
                    List<SynthesisPlanScore> scoreList = synthesisPlanScoreMapper.selectByExample(scoreExample);
                    SynthesisPlanScore synthesisPlanScore = scoreList.get(0);
                    synthesisPlanScore.setScore(teacherScore);
                    if(teacherScore.compareTo(BigDecimal.ZERO)>0){
                        synthesisPlanScore.setStatus("答对");
                    }else {
                        synthesisPlanScore.setStatus("答错");
                    }
                    synthesisPlanScoreMapper.updateByPrimaryKey(synthesisPlanScore);
                } else if ("投资规划".equals(type)) {
                    //根据caseid和userId查询case_Financial_Ratios表
                    CaseInvestmentPlanExample example = new CaseInvestmentPlanExample();
                    example.createCriteria().andCaseIdEqualTo(caseid).andUserIdEqualTo(userid);
                    List<CaseInvestmentPlan> list = caseInvestmentPlanMapper.selectByExample(example);
                    //如果不存在 则跳过
                    if (list.isEmpty()) {
                        continue;
                    }
                    CaseInvestmentPlan stuPlan = list.get(0);
                    String analysis = stuPlan.getAnalysis();
                    BigDecimal teacherScore = BigDecimal.ZERO;
                    if (StringUtils.isBlank(analysis)) {
                        stuPlan.setAnalysisTeacherScore(teacherScore);
                    } else {
                        if (analysis.length() < 20) {
                            stuPlan.setAnalysisTeacherScore(teacherScore);
                        } else if (analysis.length() >= 50) {
                            teacherScore = BigDecimal.valueOf(2);
                            stuPlan.setAnalysisTeacherScore(teacherScore);
                        } else {
                            teacherScore = BigDecimal.valueOf(1);
                            stuPlan.setAnalysisTeacherScore(teacherScore);
                        }
                    }
                    caseInvestmentPlanMapper.updateByPrimaryKey(stuPlan);
                    //并设置综合规划成绩表 修改分数
                    SynthesisPlanScoreExample scoreExample = new SynthesisPlanScoreExample();
                    scoreExample.createCriteria().andCaseidEqualTo(caseid).andUseridEqualTo(userid).andManageMoneyMattersTypeEqualTo("投资规划").andTestCenterEqualTo("投资规划分析");
                    List<SynthesisPlanScore> scoreList = synthesisPlanScoreMapper.selectByExample(scoreExample);
                    SynthesisPlanScore synthesisPlanScore = scoreList.get(0);
                    synthesisPlanScore.setScore(teacherScore);
                    if(teacherScore.compareTo(BigDecimal.ZERO)>0){
                        synthesisPlanScore.setStatus("答对");
                    }else {
                        synthesisPlanScore.setStatus("答错");
                    }
                    synthesisPlanScoreMapper.updateByPrimaryKey(synthesisPlanScore);
                } else if ("税务筹划".equals(type)) {
                    //根据caseid和userId查询case_Financial_Ratios表
                    CaseTaxPlanExample example = new CaseTaxPlanExample();
                    example.createCriteria().andCaseidEqualTo(caseid).andUserIdEqualTo(userid);
                    List<CaseTaxPlan> list = caseTaxPlanMapper.selectByExample(example);
                    //如果不存在 则跳过
                    if (list.isEmpty()) {
                        continue;
                    }
                    CaseTaxPlan stuPlan = list.get(0);
                    String analysis = stuPlan.getAnalysis();
                    BigDecimal teacherScore = BigDecimal.ZERO;
                    if (StringUtils.isBlank(analysis)) {
                        stuPlan.setAnalysisTeacherScore(teacherScore);
                    } else {
                        if (analysis.length() < 20) {
                            stuPlan.setAnalysisTeacherScore(teacherScore);
                        } else if (analysis.length() >= 50) {
                            teacherScore = BigDecimal.valueOf(2);
                            stuPlan.setAnalysisTeacherScore(teacherScore);
                        } else {
                            teacherScore = BigDecimal.valueOf(1);
                            stuPlan.setAnalysisTeacherScore(teacherScore);
                        }
                    }
                    caseTaxPlanMapper.updateByPrimaryKey(stuPlan);
                    //并设置综合规划成绩表 修改分数
                    SynthesisPlanScoreExample scoreExample = new SynthesisPlanScoreExample();
                    scoreExample.createCriteria().andCaseidEqualTo(caseid).andUseridEqualTo(userid).andManageMoneyMattersTypeEqualTo("税务筹划").andTestCenterEqualTo("税务筹划分析");
                    List<SynthesisPlanScore> scoreList = synthesisPlanScoreMapper.selectByExample(scoreExample);
                    SynthesisPlanScore synthesisPlanScore = scoreList.get(0);
                    synthesisPlanScore.setScore(teacherScore);
                    if(teacherScore.compareTo(BigDecimal.ZERO)>0){
                        synthesisPlanScore.setStatus("答对");
                    }else {
                        synthesisPlanScore.setStatus("答错");
                    }
                    synthesisPlanScoreMapper.updateByPrimaryKey(synthesisPlanScore);
                } else if ("财产分配".equals(type)) {
                    //根据caseid和userId查询case_Financial_Ratios表
                    CaseDistributionOfPropertyExample example = new CaseDistributionOfPropertyExample();
                    example.createCriteria().andCaseidEqualTo(caseid).andUserIdEqualTo(userid);
                    List<CaseDistributionOfPropertyWithBLOBs> list = caseDistributionOfPropertyMapper.selectByExampleWithBLOBs(example);
                    //如果不存在 则跳过
                    if (list.isEmpty()) {
                        continue;
                    }
                    CaseDistributionOfPropertyWithBLOBs stuPlan = list.get(0);
                    String analysis1 = stuPlan.getSituationAnalysis();
                    BigDecimal teacherScore1 = BigDecimal.ZERO;
                    if (StringUtils.isBlank(analysis1)) {
                        stuPlan.setSituationanalysisTeacherScore(teacherScore1);
                    } else {
                        if (analysis1.length() < 20) {
                            stuPlan.setSituationanalysisTeacherScore(teacherScore1);
                        } else if (analysis1.length() >= 50) {
                            teacherScore1 = BigDecimal.valueOf(2);
                            stuPlan.setSituationanalysisTeacherScore(teacherScore1);
                        } else {
                            teacherScore1 = BigDecimal.valueOf(1);
                            stuPlan.setSituationanalysisTeacherScore(teacherScore1);
                        }
                    }
                    String analysis2 = stuPlan.getPlanAnalysis();
                    BigDecimal teacherScore2 = BigDecimal.ZERO;
                    if (StringUtils.isBlank(analysis2)) {
                        stuPlan.setPlananalysisTeacherScore(teacherScore2);
                    } else {
                        if (analysis2.length() < 20) {
                            stuPlan.setPlananalysisTeacherScore(teacherScore2);
                        } else if (analysis2.length() >= 50) {
                            teacherScore2 = BigDecimal.valueOf(2);
                            stuPlan.setPlananalysisTeacherScore(teacherScore2);
                        } else {
                            teacherScore2 = BigDecimal.valueOf(1);
                            stuPlan.setPlananalysisTeacherScore(teacherScore2);
                        }
                    }
                    caseDistributionOfPropertyMapper.updateByPrimaryKey(stuPlan);
                    //并设置综合规划成绩表 修改分数
                    SynthesisPlanScoreExample scoreExample = new SynthesisPlanScoreExample();
                    scoreExample.createCriteria().andCaseidEqualTo(caseid).andUseridEqualTo(userid).andManageMoneyMattersTypeEqualTo("财产分配").andTestCenterEqualTo("婚姻、财产状况分析");
                    List<SynthesisPlanScore> scoreList = synthesisPlanScoreMapper.selectByExample(scoreExample);
                    SynthesisPlanScore synthesisPlanScore = scoreList.get(0);
                    synthesisPlanScore.setScore(teacherScore1);
                    if(teacherScore1.compareTo(BigDecimal.ZERO)>0){
                        synthesisPlanScore.setStatus("答对");
                    }else {
                        synthesisPlanScore.setStatus("答错");
                    }
                    synthesisPlanScoreMapper.updateByPrimaryKey(synthesisPlanScore);
                    SynthesisPlanScoreExample scoreExample2 = new SynthesisPlanScoreExample();
                    scoreExample2.createCriteria().andCaseidEqualTo(caseid).andUseridEqualTo(userid).andManageMoneyMattersTypeEqualTo("财产分配").andTestCenterEqualTo("财产分配规划分析");
                    List<SynthesisPlanScore> scoreList2 = synthesisPlanScoreMapper.selectByExample(scoreExample2);
                    SynthesisPlanScore synthesisPlanScore2 = scoreList2.get(0);
                    synthesisPlanScore2.setScore(teacherScore2);
                    if(teacherScore2.compareTo(BigDecimal.ZERO)>0){
                        synthesisPlanScore2.setStatus("答对");
                    }else {
                        synthesisPlanScore2.setStatus("答错");
                    }
                    synthesisPlanScoreMapper.updateByPrimaryKey(synthesisPlanScore2);
                } else if ("财产传承".equals(type)) {
                    //根据caseid和userId查询case_Financial_Ratios表
                    CaseHeritageExample example = new CaseHeritageExample();
                    example.createCriteria().andCaseidEqualTo(caseid).andUserIdEqualTo(userid);
                    List<CaseHeritageWithBLOBs> list = caseHeritageMapper.selectByExampleWithBLOBs(example);
                    //如果不存在 则跳过
                    if (list.isEmpty()) {
                        continue;
                    }
                    CaseHeritageWithBLOBs stuPlan = list.get(0);
                    String analysis2 = stuPlan.getPlanAnalysis();
                    BigDecimal teacherScore2 = BigDecimal.ZERO;
                    if (StringUtils.isBlank(analysis2)) {
                        stuPlan.setPlananalysisTeacherScore(teacherScore2);
                    } else {
                        if (analysis2.length() < 20) {
                            stuPlan.setPlananalysisTeacherScore(teacherScore2);
                        } else if (analysis2.length() >= 50) {
                            teacherScore2 = BigDecimal.valueOf(2);
                            stuPlan.setPlananalysisTeacherScore(teacherScore2);
                        } else {
                            teacherScore2 = BigDecimal.valueOf(1);
                            stuPlan.setPlananalysisTeacherScore(teacherScore2);
                        }
                    }
                    caseHeritageMapper.updateByPrimaryKey(stuPlan);
                    //并设置综合规划成绩表 修改分数
                    SynthesisPlanScoreExample scoreExample2 = new SynthesisPlanScoreExample();
                    scoreExample2.createCriteria().andCaseidEqualTo(caseid).andUseridEqualTo(userid).andManageMoneyMattersTypeEqualTo("财产传承").andTestCenterEqualTo("财产传承规划分析");
                    List<SynthesisPlanScore> scoreList2 = synthesisPlanScoreMapper.selectByExample(scoreExample2);
                    SynthesisPlanScore synthesisPlanScore2 = scoreList2.get(0);
                    synthesisPlanScore2.setScore(teacherScore2);
                    if(teacherScore2.compareTo(BigDecimal.ZERO)>0){
                        synthesisPlanScore2.setStatus("答对");
                    }else {
                        synthesisPlanScore2.setStatus("答错");
                    }
                    synthesisPlanScoreMapper.updateByPrimaryKey(synthesisPlanScore2);
                } else {
                    continue;
                }
            }
            client.setSubmitStatus("已审核");
            synthesisPlanClientMapper.updateByPrimaryKeySelective(client);
        }
        return new ResultEntity<HttpStatus>(HttpStatus.OK, "一键评分成功");
    }

    //计算理财方案
    public BigDecimal getFinancialPlanningMoney(BigDecimal returnOnInvestment, BigDecimal disposableInput, BigDecimal monthlyInvestment, Integer regularYear) {
        // returnOnInvestment 预计投资收益率 r
        // disposableInput 一次性投入金额 pv
        // monthlyInvestment 每月定期投资金额 pmt
        // regularYear 定期定额投资年份
        Integer regularMonth = regularYear * 12;  //定期定额投资月份 n
        //单笔投资的终值  FV=PV*（1+r）^n FV终值 PV一次性投入金额 r预计投资收益率 n定期定额投资月份
        BigDecimal dbFV = disposableInput.multiply((BigDecimal.ONE.add(returnOnInvestment)).pow(regularMonth)).setScale(2,BigDecimal.ROUND_HALF_UP);
        //普通年金的终值   FV=PMT*(((1+r)^n)-1)/r   PMT 每月定期投资金额 r预计投资收益率 n定期定额投资月份
        BigDecimal ptFV = monthlyInvestment.multiply(((((BigDecimal.ONE.add(returnOnInvestment)).pow(regularMonth)).subtract(BigDecimal.ONE)).divide(returnOnInvestment)));
//        BigDecimal pow = BigDecimal.ONE.add(returnOnInvestment).pow(regularMonth);
//        BigDecimal subtract = pow.subtract(BigDecimal.ONE);
//        BigDecimal divide = monthlyInvestment.multiply(subtract);
//        divide
        BigDecimal answer = dbFV.add(ptFV).setScale(2,BigDecimal.ROUND_HALF_UP);
        return answer;
    }


    //教师端 理论考核查询
    @AnonymousAccess
    @ApiOperation("教师端-成绩中心-理论考核查询")
    @PostMapping("selectTeaTheoryScore")
    public ResultEntity<PageInfo<StuTheoryRecord>> selectTeaTheoryScore(@RequestParam String schoolId, @RequestParam Integer index, @RequestParam Integer size,
                                                                        @RequestParam(required = false) String name,
                                                                        @RequestParam(required = false) String studentId,
                                                                        @RequestParam(required = false) String classId
    ) {
        PageHelper.startPage(index, size);
        StuTheoryRecordExample theoryRecordExample = new StuTheoryRecordExample();
        StuTheoryRecordExample.Criteria criteria = theoryRecordExample.createCriteria();
        criteria.andSchoolIdEqualTo(schoolId);
        if (StringUtils.isNotBlank(name)) {
            criteria.andNameEqualTo(name);
        }
        if (StringUtils.isNotBlank(studentId)) {
            criteria.andSchoolIdEqualTo(studentId);
        }
        if (StringUtils.isNotBlank(classId)) {
            criteria.andClassIdEqualTo(classId);
        }
        theoryRecordExample.setOrderByClause("average_score desc");
        List<StuTheoryRecord> stuTheoryRecords = stuTheoryRecordMapper.selectByExample(theoryRecordExample);
        if (stuTheoryRecords.isEmpty()) {
            return null;
        }
        PageInfo pageInfo = new PageInfo(stuTheoryRecords);
        return new ResultEntity<PageInfo<StuTheoryRecord>>(pageInfo);
    }

    //教师端 理论考核导出
    @AnonymousAccess
    @GetMapping("/exportTheoryScore")
    @ApiOperation("教师端--理论考核页面导出")
    public void exportTheoryScore(HttpServletResponse response,
                                  @RequestParam String schoolId) {
        List<StuTheoryRecord> stuTheoryRecordList = scoreRankService.selectExportTheoryScoreRankList(schoolId);
        //导出的表名
        String title = IdUtil.simpleUUID();
        //表中第一行表头字段
        String[] headers = {"排名", "姓名", "学号", "平均成绩", "班级", "考试次数", "考试最高分", "考试最低分", "考试合计时长"};
        //具体需要写入excel需要哪些字段，这些字段取自UserReward类，也就是上面的实际数据结果集的泛型
        List<String> listColumn = Arrays.asList("classId", "name", "studentId", "averageScore", "className", "examCount", "highestScore", "lowestScore", "totalDuration");
        try {
            FilePortUtil.exportExcel(response, title, headers, stuTheoryRecordList, listColumn);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


}
