package com.sztzjy.money_management.controller;

import com.github.pagehelper.PageInfo;
import com.sztzjy.money_management.annotation.AnonymousAccess;
import com.sztzjy.money_management.entity.*;
import com.sztzjy.money_management.entity.dto.StuAuthTestDto;
import com.sztzjy.money_management.entity.dto.StuTheoryExamDetailDto;
import com.sztzjy.money_management.entity.dto.StuTheoryTestDto;
import com.sztzjy.money_management.entity.dto.StuTheoryTestRecordDto;
import com.sztzjy.money_management.mapper.*;
import com.sztzjy.money_management.service.TheoryTestService;
import com.sztzjy.money_management.util.ResultEntity;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.math.BigDecimal;
import java.util.*;

/**
 * @Author xcj
 * @Date 2024/8/21
 */
@RestController
@Api(tags = "理论考试相关")
@RequestMapping("api/theoryTest")
public class TheoryTestController {
    @Autowired
    private TheoryTestService theoryTestService;
    @Autowired
    private ObjectiveQuestionMapper objectiveQuestionMapper;
    @Autowired
    private StuTheoryRecordMapper recordMapper;
    @Autowired
    private UserInfoMapper userMapper;
    @Autowired
    private CaseRiskIndexMapper caseRiskIndexMapper;
    @Autowired
    private StuTheoryExamMapper stuTheoryExamMapper;


    //开始考试  选择 35道单选，每题2分，5道多选每题4分，10道判断每题1分
    @PostMapping("startTheoryTest")
    @ApiOperation("理论考试/认证考试 开始考试")
    @AnonymousAccess
    public ResultEntity<List<ObjectiveQuestionWithBLOBs>> startTheoryTest(@RequestParam String userId,
                                                                          @RequestParam String schoolId,
                                                                          @ApiParam("理论考试/理论知识/专业能力/公共基础/个人理财/风险管理/公司信贷/个人贷款") @RequestParam String source) {
        List<ObjectiveQuestionWithBLOBs> list = null;
        try {
            list = theoryTestService.startTheoryTest(userId, schoolId, source);
        } catch (IOException e) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "服务器异常，请联系管理员");
        }
        for (int i = 0; i < list.size(); i++) {
            list.get(i).setSource(String.valueOf(i + 1));
        }
        return new ResultEntity<>(HttpStatus.OK, "开始考试、客观题列表查询成功", list);
    }

    //获取理论考试开始时间
    @GetMapping("getStartTime")
    @ApiOperation("理论考试/认证考试 获取考试开始时间")
    @AnonymousAccess
    public ResultEntity<Date> getStartTime(@RequestParam String userId,
                                           @RequestParam String source) {
        Date date = theoryTestService.getStartTime(userId, source);
        return new ResultEntity<>(HttpStatus.OK, "获取理论考试开始时间成功", date);
    }


    //结束考试
    @PostMapping("endTheoryTest")
    @ApiOperation("理论考试/认证考试 结束考试 学生自动点击交卷,前端检查时间到了也发送请求")
    @AnonymousAccess
    public ResultEntity<HttpStatus> endTheoryTest(@RequestBody StuTheoryTestDto theoryTestDto) {
        int flag = theoryTestService.endTheoryTest(theoryTestDto);
        if (flag == 1) {
            return new ResultEntity<>(HttpStatus.OK, "提交试卷成功");
        } else {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "提交试卷失败");
        }
    }


    //获取理论考试开始时间
    @PostMapping("getAuthTestDropDown")
    @ApiOperation("认证考试/考试记录下拉框")
    @AnonymousAccess
    public ResultEntity<Map<Date, String>> getAuthTestDropDown(@RequestParam String userId,
                                                               @RequestParam String source) {
        StuTheoryExamExample example = new StuTheoryExamExample();
        example.createCriteria().andUserIdEqualTo(userId).andSourceEqualTo(source);
        example.setOrderByClause("exam_time desc");
        List<StuTheoryExamWithBLOBs> stuTheoryExamWithBLOBs = stuTheoryExamMapper.selectByExampleWithBLOBs(example);
        if (stuTheoryExamWithBLOBs == null || stuTheoryExamWithBLOBs.isEmpty()) {
            return null;
        }
        Map<Date, String> map = new HashMap();
        for (int i = 0; i < stuTheoryExamWithBLOBs.size(); i++) {
            StuTheoryExamWithBLOBs data = stuTheoryExamWithBLOBs.get(i);
            map.put(data.getExamTime(), data.getTheoryExamId());
        }
        return new ResultEntity<>(map);
    }


//    @PostMapping("getAuthTestRecord")
//    @ApiOperation("认证考试/获取考试记录")
//    @AnonymousAccess
//    public ResultEntity<PageInfo<StuAuthTestDto>> getAuthTestRecord(@RequestParam Integer index,
//                                                                    @RequestParam Integer size,
//                                                                    @RequestParam String userId,
//                                                                    @RequestParam String source) {
//        return theoryTestService.getAuthTestRecord(index, size, userId, source);
//    }


    //错题集展示
    @GetMapping("selectErrors")
    @ApiOperation("理论考试/错题集展示")
    @AnonymousAccess
    public ResultEntity<List<ObjectiveQuestionWithBLOBs>> selectErrors(@RequestParam String userId,
                                                                       @RequestParam String schoolId) {
        List<ObjectiveQuestionWithBLOBs> list = null;
        list = theoryTestService.selectErrors(userId, schoolId);
        if (list == null) {
            return new ResultEntity<>(HttpStatus.OK, "没有错题", null);
        }
        for (int i = 0; i < list.size(); i++) {
            list.get(i).setSource(String.valueOf(i + 1));
        }
        return new ResultEntity<>(HttpStatus.OK, "错题集展示成功", list);
    }


    //移除错题集
    @GetMapping("removeError")
    @ApiOperation("理论考试/移除错题集")
    @AnonymousAccess
    public ResultEntity<HttpStatus> removeError(@RequestParam String userId,
                                                @ApiParam("客观题id") @RequestParam String objectiveQuestionId,
                                                @ApiParam("题目类型 0、1、2") @RequestParam Integer type) {
        Boolean flag = theoryTestService.removeError(userId, objectiveQuestionId, type);
        if (flag) {
            return new ResultEntity<>(HttpStatus.OK, "移除错题集成功");
        } else {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "移除错题集失败");
        }
    }


    //考试记录查询
    @PostMapping("getTheoryTestList")
    @ApiOperation("理论考试/获取理论考试数据集")
    @AnonymousAccess
    public ResultEntity<PageInfo<StuTheoryExamWithBLOBs>> getTheoryTestList(@RequestParam String userId,
                                                                            @RequestParam Integer index,
                                                                            @RequestParam Integer size) {
        PageInfo<StuTheoryExamWithBLOBs> pageInfo = theoryTestService.getTheoryTestList(userId, index, size);
        return new ResultEntity<>(HttpStatus.OK, "获取理论考试数据集成功", pageInfo);
    }


    //考试记录详情查询
    @GetMapping("getTheoryTestDetail")
    @ApiOperation("理论考试/认证考试/获取考试数据详情")
    @AnonymousAccess
    public ResultEntity<List<StuTheoryExamDetailDto>> getTheoryTestDetail(@ApiParam("理论考试id") @RequestParam String theoryExamId,
                                                                          @RequestParam Integer index,
                                                                          @RequestParam Integer size) {
        List<StuTheoryExamDetailDto> detailDtoList = null;
        try {
            detailDtoList = theoryTestService.getTheoryTestDetail(theoryExamId, index, size);
        } catch (Exception e) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "服务器异常，请联系管理员");
        }
        return new ResultEntity<>(HttpStatus.OK, "获取理论考试数据详情成功", detailDtoList);
    }


//    答题次数、平均正确率、排名（理论考试记录） ************
    @GetMapping("getTheoryRecord")
    @ApiOperation("理论考试/获取考试记录")
    @AnonymousAccess
    public ResultEntity<StuTheoryTestRecordDto> getTheoryRecord(String userId) {
        //先根据userid查询是否有记录 没有记录返回null
        StuTheoryRecord record = recordMapper.selectByPrimaryKey(userId);
        if (record == null) {
            StuTheoryTestRecordDto stuTheoryRecordDto = new StuTheoryTestRecordDto();
            stuTheoryRecordDto.setExamCount(0);
            stuTheoryRecordDto.setAverageScore(BigDecimal.ZERO);
            return new ResultEntity<>(HttpStatus.OK, "暂无理论考试记录", stuTheoryRecordDto);
        }
        UserInfo stuUser = userMapper.selectByPrimaryKey(userId);
        String schoolId = stuUser.getSchoolId();
        //查询该学校下的所有平均分 按高低排序
        StuTheoryRecordExample example = new StuTheoryRecordExample();
        example.createCriteria().andSchoolIdEqualTo(schoolId);
        example.setOrderByClause("average_score desc");
        List<StuTheoryRecord> stuTheoryRecords = recordMapper.selectByExample(example);
        Integer paiming = null;
        if (stuTheoryRecords.size() == 0) {
            StuTheoryTestRecordDto stuTheoryRecordDto = new StuTheoryTestRecordDto();
            stuTheoryRecordDto.setExamCount(0);
            stuTheoryRecordDto.setAverageScore(BigDecimal.ZERO);
            stuTheoryRecordDto.setRank(0);

            return new ResultEntity<>(HttpStatus.OK, "获取理论考试记录数据成功", stuTheoryRecordDto);
        } else {
            for (int i = 0; i < stuTheoryRecords.size(); i++) {
                if (stuTheoryRecords.get(i).getUserId().equals(userId)) {
                    paiming = i + 1;
                }
            }
        }
        StuTheoryRecord stuTheoryRecord = recordMapper.selectByPrimaryKey(userId);
        StuTheoryTestRecordDto stuTheoryRecordDto = new StuTheoryTestRecordDto();
        stuTheoryRecordDto.setExamCount(stuTheoryRecord.getExamCount());
        stuTheoryRecordDto.setAverageScore(stuTheoryRecord.getAverageScore());
        stuTheoryRecordDto.setRank(paiming);

        return new ResultEntity<>(HttpStatus.OK, "获取理论考试记录数据成功", stuTheoryRecordDto);
    }


    //获取题库全部数据
    @PostMapping("getObjQuestionCount")
    @ApiOperation("理论考试/获取客观题库数量")
    @AnonymousAccess
    public ResultEntity<HashMap<String, Long>> getObjQuestionCount(@RequestParam String schoolId) {
        HashMap<String, Long> hashMap = new HashMap<>();
        long single = getObjQuestionCountByType(0, schoolId);
        long many = getObjQuestionCountByType(1, schoolId);
        long judgment = getObjQuestionCountByType(2, schoolId);
        hashMap.put("单选题", single);
        hashMap.put("多选题", many);
        hashMap.put("判断题", judgment);
        return new ResultEntity<>(HttpStatus.OK, "获取客观题库数量成功", hashMap);
    }

    private long getObjQuestionCountByType(int i, String schoolId) {
        ObjectiveQuestionExample objectiveQuestionExample = new ObjectiveQuestionExample();
        List<String> list = new ArrayList<>();
        list.add(schoolId);
        list.add("999999999");
        objectiveQuestionExample.createCriteria().andTypeEqualTo(i).andOutlineEqualTo("普通题库").andSourceIn(list);
        return objectiveQuestionMapper.countByExample(objectiveQuestionExample);
    }


}
